# -*- coding: utf-8 -*-
"""
改进的消息调度器
支持线程池 + 异步任务池的混合架构
"""

import asyncio
import os
import threading
import time
from concurrent.futures import ThreadPoolExecutor
from typing import Callable, Awaitable, Dict, Any, Optional
from agentcp.base.log import log_info, log_error, log_exception, log_warning, log_debug


def _parse_positive_int_env(name: str) -> Optional[int]:
    val = os.getenv(name)
    if not val:
        return None
    try:
        parsed = int(val)
    except ValueError:
        log_warning(f"[Scheduler] {name} 非法值: {val}, 已忽略")
        return None
    if parsed <= 0:
        log_warning(f"[Scheduler] {name} 需要为正整数: {val}, 已忽略")
        return None
    return parsed


def _default_core_workers() -> int:
    cpu = os.cpu_count() or 1
    return min(8, max(2, cpu // 2))


def _default_max_workers(core_workers: int) -> int:
    return core_workers * 2


class ImprovedMessageScheduler:
    """
    改进的消息调度器

    架构:
    - 核心线程: core_workers 个 (常驻)
    - 最大线程: max_workers 个 (高峰扩展)
    - 每线程并发异步任务: max_tasks_per_worker 个

    优势:
    - 资源高效: 少量线程处理大量异步任务
    - 高并发: 总并发 = max_workers × max_tasks_per_worker
    - 负载均衡: 自动选择负载最低的工作线程
    """

    def __init__(self,
                 core_workers: Optional[int] = None,
                 max_workers: Optional[int] = None,
                 max_tasks_per_worker: Optional[int] = None):
        """
        初始化调度器

        Args:
            core_workers: 核心工作线程数 (常驻). None/<=0 时使用环境变量或默认值
            max_workers: 最大工作线程数 (高峰时扩展). None/<=0 时使用环境变量或默认值
            max_tasks_per_worker: 每个工作线程的最大并发异步任务数. None/<=0 时使用环境变量或默认值
        """
        orig_core = core_workers
        orig_max = max_workers
        orig_max_tasks = max_tasks_per_worker

        env_core = _parse_positive_int_env("AGENTCP_SCHED_CORE_WORKERS")
        env_max = _parse_positive_int_env("AGENTCP_SCHED_MAX_WORKERS")
        env_max_tasks = _parse_positive_int_env("AGENTCP_SCHED_MAX_TASKS_PER_WORKER")

        if core_workers is None or core_workers <= 0:
            core_workers = env_core if env_core is not None else _default_core_workers()
        if max_workers is None or max_workers <= 0:
            max_workers = env_max if env_max is not None else _default_max_workers(core_workers)
        if max_tasks_per_worker is None or max_tasks_per_worker <= 0:
            max_tasks_per_worker = env_max_tasks if env_max_tasks is not None else 10

        if max_workers < core_workers:
            log_warning(
                f"[Scheduler] max_workers ({max_workers}) < core_workers ({core_workers}), "
                f"已调整 max_workers={core_workers}"
            )
            max_workers = core_workers

        source_core = "arg" if orig_core is not None and orig_core > 0 else ("env" if env_core is not None else "cpu_default")
        source_max = "arg" if orig_max is not None and orig_max > 0 else ("env" if env_max is not None else "derived_default")
        source_tasks = "arg" if orig_max_tasks is not None and orig_max_tasks > 0 else ("env" if env_max_tasks is not None else "default")
        log_info(
            "[Scheduler] 自检: "
            f"core_workers={core_workers}({source_core}), "
            f"max_workers={max_workers}({source_max}), "
            f"max_tasks_per_worker={max_tasks_per_worker}({source_tasks})"
        )

        self.core_workers = core_workers
        self.max_workers = max_workers
        self.max_tasks_per_worker = max_tasks_per_worker

        # 线程池
        self.thread_pool = ThreadPoolExecutor(
            max_workers=max_workers,
            thread_name_prefix="agentcp-worker"
        )

        # 工作线程状态
        self.worker_loops: Dict[int, asyncio.AbstractEventLoop] = {}  # worker_id -> event loop
        self.worker_queues: Dict[int, asyncio.Queue] = {}  # worker_id -> message queue
        self.worker_tasks_count: Dict[int, int] = {}  # worker_id -> active task count
        self.worker_lock = threading.Lock()

        # 统计信息
        self.total_messages = 0
        self.total_processed = 0
        self.total_errors = 0
        self.total_rejected = 0  # ✅ P1修复: 添加拒绝计数
        self.active_workers = 0
        self.is_running = True

        # ✅ 修复: 添加统计锁,保护计数器的线程安全
        self.stats_lock = threading.Lock()

        # ✅ P1修复: 队列监控配置
        self.queue_warn_threshold = 0.8  # 队列使用率警告阈值
        self.queue_timeout = 5.0  # ✅ 增加: 队列等待超时从2秒增加到5秒
        self.max_submit_retries = 3  # ✅ 新增: 提交失败时的最大重试次数

        # 初始化核心工作线程
        self._init_core_workers()

        log_info(f"[Scheduler] 初始化完成: core_workers={core_workers}, "
                f"max_workers={max_workers}, max_tasks_per_worker={max_tasks_per_worker}")

    def _init_core_workers(self):
        """初始化核心工作线程"""
        for worker_id in range(self.core_workers):
            self._start_worker(worker_id)

    def _start_worker(self, worker_id: int):
        """
        ✅ P1修复: 启动一个工作线程 (使用Event同步)

        Args:
            worker_id: 工作线程ID
        """
        # 创建启动就绪事件
        ready_event = threading.Event()

        # 提交工作线程任务
        self.thread_pool.submit(self._worker_main, worker_id, ready_event)

        # ✅ 使用Event等待线程就绪 (最多等待5秒)
        if not ready_event.wait(timeout=5.0):
            log_error(f"[Worker-{worker_id}] 启动超时")
            raise RuntimeError(f"Worker-{worker_id} failed to start within 5 seconds")

        with self.worker_lock:
            self.worker_tasks_count[worker_id] = 0
            self.active_workers += 1

        log_info(f"[Worker-{worker_id}] 启动成功")

    def _worker_main(self, worker_id: int, ready_event: threading.Event = None):
        """
        ✅ P1修复: 工作线程主函数
        运行一个持久的事件循环,处理异步任务

        Args:
            worker_id: 工作线程ID
            ready_event: 启动就绪事件 (用于同步启动)
        """
        # 创建新的事件循环
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)

        # ✅ 增加: 队列大小从1000增加到5000，提高并发容量
        queue = asyncio.Queue(maxsize=5000)

        # 注册到全局状态
        with self.worker_lock:
            self.worker_loops[worker_id] = loop
            self.worker_queues[worker_id] = queue

        log_info(f"[Worker-{worker_id}] 事件循环启动, thread={threading.current_thread().name}")

        # ✅ 通知启动完成
        if ready_event:
            ready_event.set()

        # 运行事件循环
        try:
            loop.run_until_complete(self._worker_loop(worker_id, queue))
        except Exception as e:
            log_exception(f"[Worker-{worker_id}] 事件循环异常: {e}")
        finally:
            # 清理
            try:
                # 取消所有待处理任务
                pending = asyncio.all_tasks(loop)
                for task in pending:
                    task.cancel()
                # 等待取消完成
                loop.run_until_complete(asyncio.gather(*pending, return_exceptions=True))
                loop.close()
            except Exception as e:
                log_error(f"[Worker-{worker_id}] 清理异常: {e}")

            # 从全局状态移除
            with self.worker_lock:
                if worker_id in self.worker_loops:
                    del self.worker_loops[worker_id]
                if worker_id in self.worker_queues:
                    del self.worker_queues[worker_id]
                if worker_id in self.worker_tasks_count:
                    del self.worker_tasks_count[worker_id]
                self.active_workers -= 1

            log_info(f"[Worker-{worker_id}] 已停止")

    async def _worker_loop(self, worker_id: int, queue: asyncio.Queue):
        """✅ P0-3改进: 工作线程事件循环，优化任务拒绝逻辑

        改进要点：
        1. 移除"放回队列"的逻辑（会导致死锁）
        2. 任务超限时直接跳过（让 submit_message 重试其他 worker）
        3. 添加详细的日志

        Args:
            worker_id: 工作线程ID
            queue: 消息队列
        """
        while self.is_running:
            try:
                # 等待新消息 (超时检查,避免卡死)
                try:
                    message_handler, data = await asyncio.wait_for(queue.get(), timeout=1.0)
                except asyncio.TimeoutError:
                    continue

                # ✅ 检查是否超过任务限制
                with self.worker_lock:
                    current_tasks = self.worker_tasks_count.get(worker_id, 0)

                    if current_tasks >= self.max_tasks_per_worker:
                        # ✅ P0-3改进: 不再尝试放回队列，直接记录拒绝
                        with self.stats_lock:
                            self.total_rejected += 1

                        message_id = data.get('message_id', 'unknown') if isinstance(data, dict) else 'unknown'
                        log_warning(
                            f"⚠️ [Worker-{worker_id}] 任务超限 "
                            f"({current_tasks}/{self.max_tasks_per_worker}), "
                            f"拒绝任务 message_id={message_id[:16] if len(message_id) > 16 else message_id}..."
                        )

                        # 短暂等待后继续取下一个任务
                        await asyncio.sleep(0.05)
                        continue

                    # 增加任务计数
                    self.worker_tasks_count[worker_id] = current_tasks + 1

                # ✅ 关键:创建异步任务(不等待完成)
                asyncio.create_task(
                    self._handle_message_wrapper(worker_id, message_handler, data)
                )

            except asyncio.CancelledError:
                log_info(f"[Worker-{worker_id}] 收到取消信号")
                break
            except Exception as e:
                log_exception(f"[Worker-{worker_id}] 事件循环异常: {e}")
                await asyncio.sleep(0.1)

    async def _handle_message_wrapper(self,
                                      worker_id: int,
                                      message_handler: Callable[[Dict], Awaitable[None]],
                                      data: Dict[str, Any]):
        """
        消息处理包装器
        处理完成后减少任务计数

        Args:
            worker_id: 工作线程ID
            message_handler: 异步消息处理函数
            data: 消息数据
        """
        try:
            # 调用实际的消息处理函数
            await message_handler(data)

            # ✅ 修复: 使用锁保护统计计数器
            with self.stats_lock:
                self.total_processed += 1

        except Exception as e:
            # ✅ 修复: 使用锁保护统计计数器
            with self.stats_lock:
                self.total_errors += 1
            log_exception(f"[Worker-{worker_id}] 消息处理失败: {e}")
        finally:
            # 减少任务计数
            with self.worker_lock:
                current = self.worker_tasks_count.get(worker_id, 0)
                self.worker_tasks_count[worker_id] = max(0, current - 1)

    def submit_message(self,
                       message_handler: Callable[[Dict], Awaitable[None]],
                       data: Dict[str, Any],
                       raise_on_reject: bool = False) -> bool:
        """✅ P0-3改进: 提交消息到调度器（多候选 worker + 队列监控）

        改进要点：
        1. 选择多个候选 worker 而不是单个
        2. 依次尝试候选 worker，失败时自动切换
        3. 检查队列使用率，跳过接近满的 worker
        4. 优化重试逻辑和等待时间

        Args:
            message_handler: 异步消息处理函数 async def handler(data)
            data: 消息数据
            raise_on_reject: 如果为True，在任务被拒绝时抛出异常；否则返回False

        Returns:
            bool: True表示提交成功，False表示被拒绝

        Raises:
            RuntimeError: 当raise_on_reject=True且任务被拒绝时抛出
        """
        # ✅ 统计计数
        with self.stats_lock:
            self.total_messages += 1

        # ✅ P0-3改进: 重试机制，每次尝试多个候选 worker
        last_error = None
        for retry_attempt in range(self.max_submit_retries):
            try:
                # ✅ 获取负载最低的 TOP 3 worker
                candidate_workers = self._select_workers_by_load(top_n=3)

                if not candidate_workers:
                    error_msg = "[Scheduler] 没有可用的工作线程"
                    log_error(error_msg)
                    with self.stats_lock:
                        self.total_rejected += 1
                    if raise_on_reject:
                        raise RuntimeError(error_msg)
                    return False

                # ✅ 依次尝试候选 worker
                submitted = False
                for worker_id in candidate_workers:
                    loop = self.worker_loops.get(worker_id)
                    queue = self.worker_queues.get(worker_id)

                    if not loop or not queue or loop.is_closed():
                        continue

                    # ✅ 检查队列使用率
                    queue_size = queue.qsize()
                    queue_maxsize = queue.maxsize
                    usage_rate = queue_size / queue_maxsize if queue_maxsize > 0 else 0

                    # ✅ 如果队列使用率超过 90%，跳过这个 worker
                    if usage_rate >= 0.9:
                        log_warning(
                            f"⚠️ [Worker-{worker_id}] 队列接近满 "
                            f"({queue_size}/{queue_maxsize}, {usage_rate*100:.1f}%), 尝试下一个"
                        )
                        continue

                    # ✅ 尝试提交
                    try:
                        future = asyncio.run_coroutine_threadsafe(
                            self._put_with_timeout(queue, message_handler, data),
                            loop
                        )
                        future.result(timeout=self.queue_timeout)

                        # ✅ 提交成功
                        submitted = True
                        log_debug(f"✅ [Scheduler] 消息已提交到 Worker-{worker_id}")
                        return True

                    except Exception as e:
                        log_debug(f"⚠️ [Worker-{worker_id}] 提交失败: {e}, 尝试下一个")
                        continue

                if submitted:
                    return True

                # ✅ 所有候选 worker 都失败
                last_error = "所有候选 worker 都无法接收任务"

            except Exception as e:
                last_error = str(e)

            # ✅ 重试前等待（指数退避）
            if retry_attempt < self.max_submit_retries - 1:
                wait_time = 0.05 * (2 ** retry_attempt)  # 指数退避: 0.05s, 0.1s, 0.2s
                log_warning(
                    f"⚠️ [Scheduler] 提交失败 (第{retry_attempt + 1}次), "
                    f"{wait_time}s 后重试... reason={last_error}"
                )
                time.sleep(wait_time)

        # ✅ 所有重试都失败
        with self.stats_lock:
            self.total_rejected += 1

        error_msg = f"[Scheduler] 消息提交最终失败: {last_error}"
        log_error(error_msg)

        if raise_on_reject:
            raise RuntimeError(error_msg)

        return False

    async def _put_with_timeout(self, queue: asyncio.Queue, message_handler, data):
        """
        ✅ P1修复: 带超时的队列put操作

        Args:
            queue: 目标队列
            message_handler: 消息处理器
            data: 消息数据
        """
        try:
            await asyncio.wait_for(
                queue.put((message_handler, data)),
                timeout=self.queue_timeout
            )
        except asyncio.TimeoutError:
            raise Exception(f"队列已满,等待超时 ({self.queue_timeout}s)")

    def _select_worker(self) -> Optional[int]:
        """
        选择负载最低的工作线程（保留向后兼容）

        Returns:
            worker_id 或 None (如果没有可用worker)
        """
        with self.worker_lock:
            if not self.worker_tasks_count:
                return None

            # 找到任务数最少的worker
            min_tasks = float('inf')
            selected_worker = None

            for worker_id, task_count in self.worker_tasks_count.items():
                if task_count < min_tasks:
                    min_tasks = task_count
                    selected_worker = worker_id

                    # 如果找到空闲worker,直接使用
                    if task_count == 0:
                        break

            return selected_worker

    def _select_workers_by_load(self, top_n: int = 3) -> list:
        """✅ P0-3新增: 选择负载最低的 TOP N worker

        Args:
            top_n: 返回前 N 个负载最低的 worker

        Returns:
            worker_id 列表，按负载从低到高排序
        """
        with self.worker_lock:
            if not self.worker_tasks_count:
                return []

            # 按任务数排序（从少到多）
            sorted_workers = sorted(
                self.worker_tasks_count.items(),
                key=lambda x: x[1]  # x[1] 是任务数
            )

            # 返回前 N 个 worker 的 ID
            return [worker_id for worker_id, _ in sorted_workers[:top_n]]

    def get_stats(self) -> Dict[str, Any]:
        """
        ✅ 修复: 获取统计信息 (线程安全)

        Returns:
            统计数据字典
        """
        # 获取 worker 统计
        with self.worker_lock:
            total_active_tasks = sum(self.worker_tasks_count.values())
            worker_details = dict(self.worker_tasks_count)
            active_workers = self.active_workers

        # 获取全局统计
        with self.stats_lock:
            total_messages = self.total_messages
            total_processed = self.total_processed
            total_errors = self.total_errors
            total_rejected = self.total_rejected  # ✅ P1修复: 添加拒绝统计

        return {
            'total_messages': total_messages,
            'total_processed': total_processed,
            'total_errors': total_errors,
            'total_rejected': total_rejected,  # ✅ P1修复
            'active_workers': active_workers,
            'total_active_tasks': total_active_tasks,
            'worker_tasks': worker_details,
            'success_rate': f"{(total_processed / max(1, total_messages)) * 100:.2f}%"
        }

    def print_stats(self):
        """✅ P1修复: 打印统计信息 (包含拒绝数)"""
        stats = self.get_stats()
        log_info(f"[Scheduler Stats] "
                f"Messages: {stats['total_messages']}, "
                f"Processed: {stats['total_processed']}, "
                f"Errors: {stats['total_errors']}, "
                f"Rejected: {stats['total_rejected']}, "
                f"Active Workers: {stats['active_workers']}, "
                f"Active Tasks: {stats['total_active_tasks']}, "
                f"Success Rate: {stats['success_rate']}")

    def shutdown(self, wait: bool = True):
        """
        关闭调度器

        Args:
            wait: 是否等待所有任务完成
        """
        log_info("[Scheduler] 正在关闭...")
        self.is_running = False

        if wait:
            # 等待一段时间让任务完成
            max_wait = 10  # 最多等待10秒
            for i in range(max_wait):
                stats = self.get_stats()
                if stats['total_active_tasks'] == 0:
                    break
                log_info(f"[Scheduler] 等待任务完成... 剩余 {stats['total_active_tasks']} 个")
                time.sleep(1)

        # 关闭线程池
        self.thread_pool.shutdown(wait=wait)

        # 打印最终统计
        self.print_stats()
        log_info("[Scheduler] 已关闭")



# -*- coding: utf-8 -*-
# Copyright 2025 AgentUnion Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import io
from typing import Tuple


def uint64_to_varint(v: int) -> bytes:
    buf = b''
    while v >= 0x80:
        buf += bytes([v & 0x7F | 0x80])
        v >>= 7
    buf += bytes([v])
    return buf

def varint_to_uint64(buf: bytes) -> Tuple[int, int]:
    v = 0
    shift = 0
    for i, b in enumerate(buf):
        v |= (b & 0x7F) << shift
        shift += 7
        if not (b & 0x80):
            return v, i + 1
    raise ValueError("Invalid varint")

class UdpMessageHeader:
    def __init__(self):
        self.MessageMask: int = 0
        self.MessageSeq: int = 0
        self.MessageType: int = 0
        self.PayloadSize: int = 0
    
    def serialize(self, buf: io.BytesIO) -> None:
        buf.write(uint64_to_varint(self.MessageMask))
        buf.write(uint64_to_varint(self.MessageSeq))
        buf.write(self.MessageType.to_bytes(2, 'big'))
        buf.write(self.PayloadSize.to_bytes(2, 'big'))
    
    @classmethod
    def deserialize(cls, buf: bytes, offset: int) -> Tuple['UdpMessageHeader', int]:
        obj = cls()
        obj.MessageMask, read = varint_to_uint64(buf[offset:])
        offset += read
        obj.MessageSeq, read = varint_to_uint64(buf[offset:])
        offset += read
        obj.MessageType = int.from_bytes(buf[offset:offset+2], 'big')
        offset += 2
        obj.PayloadSize = int.from_bytes(buf[offset:offset+2], 'big')
        offset += 2
        return obj, offset

# ================== HeartbeatMessageReq ==================
class HeartbeatMessageReq:
    def __init__(self):
        self.header = UdpMessageHeader()
        self.AgentId: str = ""
        self.SignCookie: int = 0
    
    def serialize(self, buf: io.BytesIO) -> None:
        self.header.serialize(buf)
        agent_id_bytes = self.AgentId.encode('utf-8')
        buf.write(uint64_to_varint(len(agent_id_bytes)))
        buf.write(agent_id_bytes)
        buf.write(self.SignCookie.to_bytes(8, 'big'))
    
    @classmethod
    def deserialize(cls, buf: bytes, offset: int) -> Tuple['HeartbeatMessageReq', int]:
        obj = cls()
        obj.header, offset = UdpMessageHeader.deserialize(buf, offset)
        len_agent_id, read = varint_to_uint64(buf[offset:])
        offset += read
        obj.AgentId = buf[offset:offset+int(len_agent_id)].decode('utf-8')
        offset += int(len_agent_id)
        obj.SignCookie = int.from_bytes(buf[offset:offset+8], 'big')
        offset += 8
        return obj, offset

# ================== HeartbeatMessageResp ==================
class HeartbeatMessageResp:
    def __init__(self):
        self.header = UdpMessageHeader()
        self.NextBeat: int = 0
    
    def serialize(self, buf: io.BytesIO) -> None:
        self.header.serialize(buf)
        buf.write(self.NextBeat.to_bytes(8, 'big'))
    
    @classmethod
    def deserialize(cls, buf: bytes, offset: int) -> Tuple['HeartbeatMessageResp', int]:
        obj = cls()
        obj.header, offset = UdpMessageHeader.deserialize(buf, offset)
        obj.NextBeat = int.from_bytes(buf[offset:offset+8], 'big')
        offset += 8
        return obj, offset

# ================== InviteMessageReq ==================
class InviteMessageReq:
    def __init__(self):
        self.header = UdpMessageHeader()
        self.InviterAgentId: str = ""
        self.InviteCode: str = ""
        self.InviteCodeExpire: int = 0
        self.SessionId: str = ""
        self.MessageServer: str = ""

    def serialize(self, buf: io.BytesIO) -> None:
        self.header.serialize(buf)
        # 序列化 InviterAgentId
        inviter_bytes = self.InviterAgentId.encode('utf-8')
        buf.write(uint64_to_varint(len(inviter_bytes)))
        buf.write(inviter_bytes)
        # 序列化 InviteCode
        invite_code_bytes = self.InviteCode.encode('utf-8')
        buf.write(uint64_to_varint(len(invite_code_bytes)))
        buf.write(invite_code_bytes)
        # 序列化 InviteCodeExpire（int64，8字节大端）
        buf.write(self.InviteCodeExpire.to_bytes(8, 'big', signed=True))
        # 序列化 SessionId
        session_id_bytes = self.SessionId.encode('utf-8')
        buf.write(uint64_to_varint(len(session_id_bytes)))
        buf.write(session_id_bytes)
        # 序列化MessageServer
        message_server_bytes = self.MessageServer.encode("utf-8")
        buf.write(uint64_to_varint(len(message_server_bytes)))
        buf.write(message_server_bytes)

    @classmethod
    def deserialize(cls, buf: bytes, offset: int) -> Tuple['InviteMessageReq', int]:
        obj = cls()
        obj.header, offset = UdpMessageHeader.deserialize(buf, offset)
        # 反序列化 InviterAgentId
        len_inviter, read = varint_to_uint64(buf[offset:])
        offset += read
        obj.InviterAgentId = buf[offset:offset+int(len_inviter)].decode('utf-8')
        offset += int(len_inviter)
        # 反序列化 InviteCode
        len_invite_code, read = varint_to_uint64(buf[offset:])
        offset += read
        obj.InviteCode = buf[offset:offset+int(len_invite_code)].decode('utf-8')
        offset += int(len_invite_code)
        # 反序列化 InviteCodeExpire
        obj.InviteCodeExpire = int.from_bytes(buf[offset:offset+8], 'big', signed=True)
        offset += 8
        # 反序列化 SessionId
        len_session_id, read = varint_to_uint64(buf[offset:])
        offset += read
        obj.SessionId = buf[offset:offset+int(len_session_id)].decode('utf-8')
        offset += int(len_session_id)
        # 反序列化MessageServer
        len_message_server, read = varint_to_uint64(buf[offset:])
        offset += read
        obj.MessageServer = buf[offset:offset+int(len_message_server)].decode('utf-8')
        offset += int(len_message_server)
        return obj, offset

# ================== InviteMessageResp ==================
class InviteMessageResp:
    def __init__(self):
        self.header = UdpMessageHeader()
        self.AgentId: str = ""
        self.SignCookie: int = 0
        self.InviterAgentId: str = ""
        self.SessionId: str = ""
    
    def serialize(self, buf: io.BytesIO) -> None:
        self.header.serialize(buf)
        agent_id_bytes = self.AgentId.encode('utf-8')
        buf.write(uint64_to_varint(len(agent_id_bytes)))
        buf.write(agent_id_bytes)
        inviter_agent_id_bytes = self.InviterAgentId.encode('utf-8')
        buf.write(uint64_to_varint(len(inviter_agent_id_bytes)))
        buf.write(inviter_agent_id_bytes)
        session_id_bytes = self.SessionId.encode('utf-8')
        buf.write(uint64_to_varint(len(session_id_bytes)))
        buf.write(session_id_bytes)
        buf.write(self.SignCookie.to_bytes(8, 'big'))
    
    @classmethod
    def deserialize(cls, buf: bytes, offset: int) -> Tuple['InviteMessageResp', int]:
        obj = cls()
        obj.header, offset = UdpMessageHeader.deserialize(buf, offset)
        len_agent_id, read = varint_to_uint64(buf[offset:])
        offset += read
        obj.AgentId = buf[offset:offset+int(len_agent_id)].decode('utf-8')
        offset += int(len_agent_id)
        len_inviter_agent_id, read = varint_to_uint64(buf[offset:])
        offset += read
        obj.InviterAgentId = buf[offset:offset+int(len_inviter_agent_id)].decode('utf-8')
        offset += int(len_inviter_agent_id)
        len_session_id, read = varint_to_uint64(buf[offset:])
        offset += read
        obj.SessionId = buf[offset:offset+int(len_session_id)].decode('utf-8')
        offset += int(len_session_id)
        obj.SignCookie = int.from_bytes(buf[offset:offset+8], 'big')
        offset += 8
        return obj, offset

if __name__ == "__main__":
    # 测试 HeartbeatMessageReq
    req = HeartbeatMessageReq()
    req.header.MessageMask = 1
    req.header.MessageSeq = 2
    req.header.MessageType = 3
    req.header.PayloadSize = 100
    req.AgentId = "agent-1"
    req.SignCookie = 12345
    buf = io.BytesIO()
    req.serialize(buf)
    data = buf.getvalue()
    deser_req, _ = HeartbeatMessageReq.deserialize(data, 0)
    
    # 测试 HeartbeatMessageResp
    resp = HeartbeatMessageResp()
    resp.header.MessageMask = 4
    resp.header.MessageSeq = 5
    resp.header.MessageType = 4
    resp.header.PayloadSize = 8
    resp.NextBeat = 60000
    buf = io.BytesIO()
    resp.serialize(buf)
    data = buf.getvalue()
    deser_resp, _ = HeartbeatMessageResp.deserialize(data, 0)
    
    # 测试 InviteMessageReq
    invite_req = InviteMessageReq()
    invite_req.header.MessageMask = 6
    invite_req.header.MessageSeq = 7
    invite_req.header.MessageType = 5
    invite_req.header.PayloadSize = 200
    invite_req.InviterAgentId = "inviter-1"
    invite_req.InviteCode = "CODE123"
    invite_req.InviteCodeExpire = 1680000000
    invite_req.SessionId = "session-1"
    buf = io.BytesIO()
    invite_req.serialize(buf)
    data = buf.getvalue()
    deser_invite_req, _ = InviteMessageReq.deserialize(data, 0)
    
    # 测试 InviteMessageResp
    invite_resp = InviteMessageResp()
    invite_resp.header.MessageMask = 8
    invite_resp.header.MessageSeq = 9
    invite_resp.header.MessageType = 6
    invite_resp.header.PayloadSize = 150
    invite_resp.AgentId = "resp-agent"
    invite_resp.SignCookie = 98765
    buf = io.BytesIO()
    invite_resp.serialize(buf)
    data = buf.getvalue()
    deser_invite_resp, _ = InviteMessageResp.deserialize(data, 0)



# -*- coding: utf-8 -*-
# Copyright 2025 AgentUnion Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from dataclasses import dataclass
from typing import Literal, Optional, Union

from agentcp.msg.open_ai_message import OpenAIMessage


@dataclass
class AgentSelectItemBlock:
    type: Literal["text", "image", "video", "audio"]
    id: str
    content: str


@dataclass
class AgentFormInputItemBlock:
    id: str
    name: str
    description: str


@dataclass
class AgentFormBlock:
    id: str
    description: str
    params: Optional[Union[list[AgentSelectItemBlock], AgentFormInputItemBlock]]
    form_type: Literal["single_select", "multiple_select", "time", "input"] = "single_select"
    result: Optional[str] = ""


@dataclass
class AgentFormItemResultBlock:
    id: str
    result: str


@dataclass
class AgentFormResultBlock:
    result: Optional[list[AgentFormItemResultBlock]] = None


@dataclass
class AgentAddFriendBlock:
    aid: str
    description: str


@dataclass
class AgentFileBlock:
    name: str
    url: str
    content: str
    description: str
    type: str
    encode: str


@dataclass
class AgentCreateOrderBlock:
    total_amount: int
    description: str
    # 发起者aid
    initiator_aid: str
    # 支付人aid，若为空则为发起者支付
    player_aid: str
    # 支付方式，如alipay,wxpay
    payment_method: str
    # 服务周期，单位为秒，如10分钟为600
    service_period: int


@dataclass
class AgentOrderBlock:
    total_amount: int
    description: str
    # 发起者aid
    initiator_aid: str
    # 支付人aid，若为空则为发起者支付
    player_aid: str
    # 支付方式，如alipay,wxpay
    payment_method: str
    # 服务周期，单位为秒，如10分钟为600
    service_period: int
    # 担保方aid
    guarantor_aid: str
    order_no: str
    status: Literal["success", "pending", "failed", "canceled"]
    order_query_url: str
    qr_code_url: str


@dataclass
class AssistantMessageBlock:
    type: Literal[
        "llm",
        "content",
        "search",
        "reasoning_content",
        "error",
        "file",
        "image",
        "tool_call",
        "text/event-stream",
        "video",
        "audio",
        "form",
        "form_result",
        "add_agent",
        "create_order",
        "order",
    ]
    status: Literal["success", "loading", "cancel", "error", "reading", "optimizing"]
    timestamp: int
    block_id: Optional[str] = None
    content: Optional[
        Union[
            str,
            AgentCreateOrderBlock,
            AgentOrderBlock,
            OpenAIMessage,
            AgentFormResultBlock,
            AgentAddFriendBlock,
            AgentFileBlock,
            list[AgentFormBlock],
        ]
    ] = None
    type_format: str = ""
    trace_id: Optional[str] = ""


@dataclass
class AgentInstructionBlock:
    cmd: str
    params: Optional[dict] = None
    description: Optional[str] = None
    model: str = ""



# -*- coding: utf-8 -*-
# Copyright 2025 AgentUnion Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import json
import asyncio
import queue
import ssl
import threading
import time
import urllib.parse
from typing import Optional

import websocket

from agentcp.base.log import log_debug, log_error, log_exception, log_info
from agentcp.utils import tls_config as tls
from agentcp.msg.wss_binary_message import *
from agentcp.utils.proxy_bypass import ensure_no_proxy_for_local_env, without_proxy_env

from ..context import ErrorContext, exceptions

ensure_no_proxy_for_local_env()

@dataclass
class FileChunk:
    offset: int
    chunk_size: int
    chunk: bytes

class StreamClient:
    def __init__(self, agent_id: str, session_id, server_url: str, signature: str):
        """消息客户端类
        Args:
            agent_id: 代理ID
            server_url: 服务器URL
        """
        self.session_id = session_id
        self.agent_id = agent_id
        self.server_url = server_url
        self.signature = signature
        self.connected_event = threading.Event()
        self.ws = None
        self.ws_url = ""
        self.ws_thread: Optional[threading.Thread] = None
        self.ws_is_running = False
        self.ws_chunks = ""
        self.file_stream_chunk_queue = queue.Queue()
        self.file_stream_push_cache_left_space = 65536

    def set_message_handler(self, message_handler):
        """设置消息处理器"""
        self.message_handler = message_handler

    async def start_websocket_client(self):
        if self.connected_event.is_set():
            return True

        # 确保URL格式正确
        ws_url = self.server_url.rstrip("/")  # 移除末尾斜杠
        ws_url = ws_url.replace("https://", "wss://").replace("http://", "ws://")
        ws_url = ws_url + f"&agent_id={self.agent_id}"

        log_debug(f"message Connecting to WebSocket URL: {ws_url}")  # 调试日志
        self.ws_url = ws_url
        self.ws_thread = threading.Thread(target=self.__ws_handler, daemon=True)
        self.ws_thread.start()
        wait = 0
        while not self.connected_event.is_set() and wait < 5:
            log_debug("WebSocket client is reconnect...")
            await asyncio.sleep(0.01)  # 使用 asyncio.sleep 避免阻塞事件循环
            wait += 0.01
        return self.connected_event.is_set()

    def send_wss_message(self, msg):
        """发送WebSocket消息，如果连接断开则尝试重连"""
        # 检查连接状态，如果断开则尝试重连
        if not self.ws or not self.ws.sock or not self.ws.sock.connected:
            try:
                # 使用事件循环在另一个线程中运行异步重连
                loop = asyncio.new_event_loop()
                def reconnect_sync():
                    asyncio.set_event_loop(loop)
                    try:
                        loop.run_until_complete(self.start_websocket_client())
                    finally:
                        loop.close()

                # 在新线程中执行重连
                reconnect_thread = threading.Thread(target=reconnect_sync, daemon=True)
                reconnect_thread.start()
                reconnect_thread.join(timeout=5.0)

                if not self.connected_event.is_set():
                    log_error("[StreamClient] WebSocket重连超时")
                    return False
            except Exception as e:
                log_error(f"[StreamClient] WebSocket重连失败: {e}")
                return False

        # 发送消息
        self.__send_wss_message_p(msg)
        return True

    def __send_wss_message_p(self, msg):
        if self.ws.sock and self.ws.sock.connected:
            bytes_msg = encode_wss_binary_message(msg)
            self.ws.send(bytes_msg, websocket.ABNF.OPCODE_BINARY)

    def stop_websocket_client(self):
        self.connected_event.clear()  # 清除连接事件
        if self.ws:
            self.ws.close()
        self.ws_thread.join()
        self.ws = None
        self.ws_thread = None

    def send_msg(self, msg):
        try:
            self.ws.send(msg)
            return True
        except Exception as e:
            log_exception(f"send message: {msg}")
            trace_id = msg.get("trace_id", "") if isinstance(msg, dict) else ""
            ErrorContext.publish(exceptions.SendMsgError(f"发送消息时发生错误: {e}", trace_id=trace_id))

    def send_chunk_to_stream(self, chunk, type="text/event-stream"):

        if self.ws is None or self.ws.sock is None or not self.ws.sock.connected:
            log_error("WebSocket connection is not established @send_chunk_to_stream.")
            self.ws_chunks += chunk
            return False
        try:
            if type=="text/event-stream":
                chunk_quote = urllib.parse.quote_plus(chunk)
                data = {
                    "cmd": "push_text_stream_req",
                    "data": {
                        "chunk": f"{chunk_quote}",
                    },
                }
                msg = json.dumps(data)
                self.send_msg(msg)  # 发送消息到 WebSocket 服务器
                return True
            return False
        except Exception as e:
            import traceback
            log_error(f"发送消息时发生错误: {str(e)}\n{traceback.format_exc()}")
            ErrorContext.publish(exceptions.SendChunkToStreamError(f"发送消息时发生错误: {e}"))
            return False

    def send_chunk_to_file_stream(self, offset: int, chunk: bytes) -> bool:
        try:
            if self.ws is None or self.ws.sock is None or not self.ws.sock.connected:
                file_chunk = FileChunk()
                file_chunk.offset = offset
                file_chunk.size = len(chunk)
                file_chunk.chunk = chunk
                self.file_stream_chunk_queue.put(file_chunk)
                time.sleep(1)
                return False
            msg = WssBinaryMessage()
            msg.magic_byte1 = ord('M')
            msg.magic_byte2 = ord('U')
            msg.version = 0x101
            msg.flags = 0
            msg.msg_type = 0x5
            msg.msg_seq = 0
            msg.content_type = 0x5
            msg.compressed = 0
            msg.reserved = offset
            if self.ws and self.ws.sock and self.ws.sock.connected:  # 检查WebSocket连接状态是否正常
                bytes_msg = encode_wss_binary_buffer(chunk, msg)
                self.ws.send(bytes_msg, websocket.ABNF.OPCODE_BINARY)
                print(f"[{time.strftime('%Y-%m-%d %H:%M:%S')}] 发送文件数据: {offset} + {len(chunk)}")
            self.file_stream_push_cache_left_space -= len(chunk)
            return self.file_stream_push_cache_left_space >= 16384
        except Exception as e:
            print(f"发送文件数据时发生错误: {str(e)}")
            return False

    def __ws_handler(self):
        """
        WebSocket客户端定时发送消息函数
        :param url: WebSocket服务器URL（ws://或wss://开头）
        :param message: 要定时发送的消息内容
        :param interval: 发送间隔时间（秒），默认5秒
        """
        try:

            def on_message(ws, message):
                """接收到服务器消息时的处理函数"""
                if isinstance(message, bytes):
                    txt_msg = decode_wss_binary_message(message)
                    if txt_msg is not None and len(txt_msg) > 0:
                        log_info(f"Text_Stream收到消息收到二进制消息: {txt_msg}")
                        on_message(ws, txt_msg)
                elif isinstance(message, str):
                    log_info(f"Text_Stream收到消息: {self.agent_id}\t{message}")

            def on_error(ws, error):
                """连接发生错误时的处理函数"""
                log_error(f"Text_Stream连接错误: {error}")
                self.connected_event.clear()  # 清除连接事件，允许重连
                ErrorContext.publish(exceptions.SDKError(f"websocket on_error: {error}"))

            def on_close(ws, close_status_code, close_msg):
                """连接关闭时的处理函数"""
                log_info("Text_Stream WebSocket 连接已关闭")
                self.connected_event.clear()  # 清除连接事件，允许重连

            def on_open(ws):
                """连接建立后的处理函数，用于发送初始消息"""
                log_info("Text_Stream WebSocket 连接已建立")
                self.connected_event.set()  # 设置连接事件
                # if self.ws_chunks is not None and len(self.ws_chunks) > 0:
                #     self.send_chunk_to_stream(self.ws_chunks)
                #     self.ws_chunks = ""

            # 创建 WebSocket 客户端实例
            self.ws = websocket.WebSocketApp(
                self.ws_url,
                on_open=on_open,
                on_message=on_message,
                on_error=on_error,
                on_close=on_close,
            )
            # 启动WebSocket连接（阻塞当前线程）
            with without_proxy_env(enabled=True):
                self.ws.run_forever(
                    sslopt=tls.websocket_sslopt(),
                )
        except Exception as e:
            log_exception(f"WebSocket连接失败: {e}")
            ErrorContext.publish(exceptions.SDKError(f"WebSocket连接失败: {e}"))

    def close_stream(self, stream_url: str):
        if self.ws and self.ws.sock and self.ws.sock.connected:  # 检查WebSocket连接状态是否正常
            data = {
                "cmd": "close_stream_req",
            }
            msg = json.dumps(data)
            self.ws.send(msg)
            # self.__send_wss_message(self.ws, msg)  # 发送消息到 WebSocket 服务器
            log_info(f"[{time.strftime('%Y-%m-%d %H:%M:%S')}] 发送消息: {msg}")

        if self.ws_is_running:
            self.ws_is_running = False
            self.ws.close()
            self.ws = None

    def __send_wss_message(self, msg):
        if self.ws.sock and self.ws.sock.connected:
            bytes_msg = encode_wss_binary_message(msg)
            self.ws.send(bytes_msg, websocket.ABNF.OPCODE_BINARY)



# -*- coding: utf-8 -*-
# Copyright 2025 AgentUnion Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from requests import get


def parse_html(json_data):
    html_content1 = """
    <!DOCTYPE html>
    <html lang="zh-CN">
    <head>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        <title>智能体描述页面</title>
        <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0/dist/css/bootstrap.min.css" rel="stylesheet">
        <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/bootstrap-icons@1.10.0/font/bootstrap-icons.css">
        <script src="https://cdn.jsdelivr.net/npm/chart.js"></script>
    <style>
            :root {
                --primary-color: #3498db;
                --secondary-color: #2ecc71;
                --dark-color: #2c3e50;
                --light-color: #ecf0f1;
            }
            
            body {
                font-family: 'Segoe UI', Tahoma, Geneva, Verdana, sans-serif;
                background-color: #f8f9fa;
                color: #333;
            }
            
            .navbar {
                background: linear-gradient(135deg, var(--primary-color), var(--dark-color));
                box-shadow: 0 4px 12px rgba(0, 0, 0, 0.1);
            }
            
            .hero-section {
                background: linear-gradient(135deg, var(--primary-color), var(--secondary-color));
                color: white;
                padding: 4rem 0;
                margin-bottom: 2rem;
                border-radius: 0 0 20px 20px;
            }
            
            .card {
                border: none;
                border-radius: 10px;
                box-shadow: 0 5px 15px rgba(0, 0, 0, 0.1);
                transition: transform 0.3s ease;
                margin-bottom: 20px;
            }
            
            .card:hover {
                transform: translateY(-5px);
            }
            
            .card-header {
                background: linear-gradient(135deg, var(--primary-color), var(--secondary-color));
                color: white;
                border-radius: 10px 10px 0 0 !important;
                font-weight: bold;
            }
            
            .feature-icon {
                font-size: 2.5rem;
                color: var(--primary-color);
                margin-bottom: 1rem;
            }
            
            .tech-specs {
                background-color: white;
                border-radius: 10px;
                padding: 2rem;
                box-shadow: 0 5px 15px rgba(0, 0, 0, 0.1);
            }
            
            .tech-specs h3 {
                color: var(--primary-color);
                margin-bottom: 1.5rem;
            }
            
            .tech-specs table {
                width: 100%;
            }
            
            .tech-specs th {
                background-color: var(--light-color);
                padding: 12px;
                text-align: left;
            }
            
            .tech-specs td {
                padding: 12px;
                border-bottom: 1px solid #eee;
            }
            
            .chart-container {
                background-color: white;
                border-radius: 10px;
                padding: 2rem;
                box-shadow: 0 5px 15px rgba(0, 0, 0, 0.1);
                margin-bottom: 2rem;
            }
            
            footer {
                background-color: var(--dark-color);
                color: white;
                padding: 2rem 0;
                margin-top: 3rem;
            }
            
            .badge-custom {
                background-color: var(--secondary-color);
                color: white;
                font-weight: normal;
                padding: 5px 10px;
                border-radius: 20px;
            }
        </style>
    </head>
    """
    html_content3 = """
    <body>
        <!-- 导航栏 -->
        <nav class="navbar navbar-expand-lg navbar-dark">
            <div class="container">
                <a class="navbar-brand" href="#">
                    <i class="bi bi-cloud-sun-fill me-2"></i>
                    {name}
                </a>
                <button class="navbar-toggler" type="button" data-bs-toggle="collapse" data-bs-target="#navbarNav">
                    <span class="navbar-toggler-icon"></span>
                </button>
                <div class="collapse navbar-collapse" id="navbarNav">
                    <ul class="navbar-nav ms-auto">
                        <li class="nav-item">
                            <a class="nav-link active" href="#overview">概述</a>
                        </li>
                        <li class="nav-item">
                            <a class="nav-link" href="#features">功能</a>
                        </li>
                        <li class="nav-item">
                            <a class="nav-link" href="#tech">技术</a>
                        </li>
                    </ul>
                </div>
            </div>
        </nav>

        <!-- 标题区 -->
        <section class="hero-section text-center">
            <div class="container">
                <h1 class="display-4 fw-bold mb-3">{name}</h1>
                <p class="lead mb-4"> {description}</p>
                <span class="badge badge-custom mb-3">版本 1.0.0</span>
                <p class="mb-0">最后更新: {lastUpdated}</p>
            </div>
        </section>

        <!-- 主要内容区 -->
        <div class="container">
            <!-- 基本信息卡片 -->
            <section id="overview" class="mb-5">
                <div class="row">
                    <div class="col-12">
                        <div class="card h-100">
                            <div class="card-header">
                                <h5 class="card-title mb-0"><i class="bi bi-info-circle me-2"></i>系统信息</h5>
                            </div>
                            <div class="card-body">
                                <ul class="list-group list-group-flush">
                                    <li class="list-group-item d-flex justify-content-between align-items-center">
                                        <span><i class="bi bi-tag me-2"></i>名称</span>
                                        <span class="text-muted">{name}</span>
                                    </li>
                                    <li class="list-group-item d-flex justify-content-between align-items-center">
                                        <span><i class="bi bi-card-text me-2"></i>发布者</span>
                                        <span class="text-muted">{publisherAid}</span>
                                    </li>
                                    <li class="list-group-item d-flex justify-content-between align-items-center">
                                        <span><i class="bi bi-123 me-2"></i>版本</span>
                                        <span class="text-muted">{version}</span>
                                    </li>
                                    <li class="list-group-item d-flex justify-content-between align-items-center">
                                        <span><i class="bi bi-calendar-check me-2"></i>最后更新</span>
                                        <span class="text-muted">{lastUpdated}</span>
                                    </li>
                                    <li class="list-group-item d-flex justify-content-between align-items-center">
                                        <span><i class="bi bi-building me-2"></i>认证机构</span>
                                        <span class="text-muted">{organization}</span>
                                    </li>
                                </ul>
                            </div>
                        </div>
                    </div>
                    <div class="col-12 mt-4">
                        <div class="card h-100">
                            <div class="card-header">
                                <h5 class="card-title mb-0"><i class="bi bi-lightning-charge me-2"></i>核心能力</h5>
                            </div>
                            <div class="card-body">
                                <div class="text-center mb-4">
                                    <i class="bi bi-geo-alt-fill feature-icon"></i>
                                    <h4>{capabilitiesCore}</h4>
                                </div>
                                <hr>
                                <div class="text-center">
                                    <i class="bi bi-arrow-repeat feature-icon"></i>
                                    <h4>异步支持</h4>
                                    <p class="text-muted">系统支持异步操作，提高响应效率</p>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </section>

            <!-- 功能展示区 -->
            <section id="features" class="mb-5">
                <h2 class="text-center mb-4" style="color: var(--primary-color);"><i class="bi bi-stars me-2"></i>功能特点</h2>
                <div class="row">
                    <div class="col-12 mb-4">
                        <div class="card h-100">
                            <div class="card-body text-center">
                                <i class="bi bi-cloud-sun feature-icon"></i>
                                <h4>{description}</h4>
                                <p class="text-muted">{capabilitiesCore}</p>
                            </div>
                        </div>
                    </div>
                    <div class="col-12 mb-4">
                        <div class="card h-100">
                            <div class="card-body text-center">
                                <i class="bi bi-code-slash feature-icon"></i>
                                <h4>标准接口</h4>
                                <p class="text-muted">采用JSON输入/Markdown输出，标准化接口设计</p>
                            </div>
                        </div>
                    </div>
                    <div class="col-12">
                        <div class="card h-100">
                            <div class="card-body text-center">
                                <i class="bi bi-shield-check feature-icon"></i>
                                <h4>安全认证</h4>
                                <p class="text-muted">通过数字签名认证，确保数据安全可靠</p>
                            </div>
                        </div>
                    </div>
                </div>
            </section>

            <!-- 技术细节表格 -->
            <section id="tech" class="mb-5">
                <div class="tech-specs">
                    <h3 class="text-center"><i class="bi bi-cpu me-2"></i>技术规格</h3>
                    <div class="table-responsive">
                        <table class="table">
                            <thead>
                                <tr>
                                    <th>类别</th>
                                    <th>项目</th>
                                    <th>详情</th>
                                </tr>
                            </thead>
                            <tbody>
                                <tr>
                                    <td rowspan="2">输入</td>
                                    <td>类型</td>
                                    <td>content</td>
                                </tr>
                                <tr>
                                    <td>格式</td>
                                    <td>JSON</td>
                                </tr>
                                <tr>
                                    <td rowspan="2">输出</td>
                                    <td>类型</td>
                                    <td>content</td>
                                </tr>
                                <tr>
                                    <td>格式</td>
                                    <td>Markdown</td>
                                </tr>
                                <tr>
                                    <td>操作模式</td>
                                    <td colspan="2">支持异步操作 (support_async: true)</td>
                                </tr>
                                <tr>
                                    <td>授权</td>
                                    <td colspan="2">免费使用 (当前智能体免费使用，无费用)</td>
                                </tr>
                                <tr>
                                    <td>认证</td>
                                    <td colspan="2">{organization}</td>
                                </tr>
                            </tbody>
                        </table>
                    </div>
                </div>
            </section>
        </div>

        <!-- 页脚 -->
        <footer class="text-center">
            <div class="container">
                <p class="mb-2"><i class="bi bi-cloud-sun-fill me-2"></i>{name}</p>
                <p class="mb-0">© 2025 {organization} 版权所有</p>
            </div>
        </footer>
    """.format(
        name=json_data.get("name", "AI Agent"),
        description=json_data.get("description","AI智能体"),
        lastUpdated=json_data.get("lastUpdated","----"),
        version=json_data.get("version","----"),
        publisherAid=json_data.get("publisherInfo",{}).get("publisherAid","未知发布者"),
        organization = json_data.get("publisherInfo",{}).get("organization","未知认证机构"),
        capabilitiesCore = "，".join(json_data.get("capabilities",{}).get("core", [])),
    )
    html_content4="""
        <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0/dist/js/bootstrap.bundle.min.js"></script>
    </body>
    </html>
    """
    return html_content1  + html_content3 +html_content4
    


# -*- coding: utf-8 -*-
# Copyright 2025 AgentUnion Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os

from agentcp.base.log import log_error, log_info
class CARoot:
    _instance = None
    
    def __new__(cls):
        if not cls._instance:
            cls._instance = super().__new__(cls)
            cls._instance.__initialized = False
        return cls._instance
    
    def set_ca_root_crt(self,ca_root_path):
        self.__ca_root_path = ca_root_path
    
    def __init__(self):
        self.__ca_crt = []
        # 内置根证书（PEM格式）
        self.__ca_crt.append("""\
-----BEGIN CERTIFICATE-----
MIICJjCCAYigAwIBAgIQf2zjuigigLrW8Su0I+2AiTAKBggqhkjOPQQDBDAnMRMw
EQYDVQQKEwpBZ2VudFVuaW9uMRAwDgYDVQQDEwdSb290IENBMB4XDTI1MDUwODA3
MDE1OFoXDTQ1MDUwODA3MDE1OFowJzETMBEGA1UEChMKQWdlbnRVbmlvbjEQMA4G
A1UEAxMHUm9vdCBDQTCBmzAQBgcqhkjOPQIBBgUrgQQAIwOBhgAEAUuDc4dMcjXI
GVxem8HStonZlyfyqtujfpTz8WP4ZcMUCTlrvnxZRjzNarmzSc2Yx2COcK1VEuzP
TcyQGE/Pw9i4AP9qGtX0j3dwLw+i2+TzEOmgoulm+t+fyjxhLsmqyWrIUdTv6T5C
IYVkSnX1mM0UPVQYxZi/2Uuyw8FOcPzIq7eWo1MwUTAOBgNVHQ8BAf8EBAMCAf4w
DwYDVR0lBAgwBgYEVR0lADAPBgNVHRMBAf8EBTADAQH/MB0GA1UdDgQWBBQhkljB
FddnHb7Y0j6YEZ7wtYReNzAKBggqhkjOPQQDBAOBiwAwgYcCQWKjy52NZwqJZ1FN
1n1BRPAIy6nFDTke+HbM/JFWGyFrNSx4ceVSurpa+Uy9TWmwNUuog82MHRDXnlYp
e1KpOe6qAkIBGGII4Yfzoc5x+ZC0le7kyAYTJJl1XVLCdwECPRfzk/uZHonFA4tV
nHMwnEqFoMdsj2GgOqoRqAw/miMQwo2T0hA=
-----END CERTIFICATE-----
""")
        self.__ca_root_path = None
        self.__initialized = True
        
    def get_ca_root_crt_number(self):
        return len(self.__ca_crt)
    
    def get_ca_root_crt(self,index=0):
        if self.__ca_root_path:
            try:
                crt_files = sorted([f for f in os.listdir(self.__ca_root_path) if f.endswith('.crt')])
                if not crt_files:
                    log_error(f"目录 {self.__ca_root_path} 中没有证书文件")
                    return self.__ca_crt[index]
                    
                if index >= len(crt_files):
                    log_error(f"索引 {index} 超出文件数量 {len(crt_files)}")
                    return None
                    
                cert_path = os.path.join(self.__ca_root_path, crt_files[index])
                with open(cert_path, 'r', encoding='utf-8') as f:
                    return f.read()
            except FileNotFoundError:
                log_error(f"根证书目录 {self.__ca_root_path} 不存在")
            except Exception as e:
                log_error(f"根读取证书文件失败: {str(e)}")
        return self.__ca_crt[index]


# -*- coding: utf-8 -*-
"""
ACP Group Client - Python implementation of group protocol.
Sends JSON requests via ACP message transport to group.{ap} server.
"""
import json
import threading
import time
import urllib.parse

from agentcp.base.log import log_info, log_error, log_warning


def send_raw_to_session(agent_id_obj, session_id, target_aid, raw_payload):
    """Send raw JSON payload via ACP session without content block wrapping."""
    session = agent_id_obj.session_manager.get(session_id)
    if not session or not session.message_client:
        raise Exception('Session not found or not connected')
    encoded = urllib.parse.quote(raw_payload)
    data = {
        "cmd": "session_message",
        "data": {
            "message_id": str(int(time.time() * 1000)),
            "session_id": session_id,
            "ref_msg_id": "",
            "sender": agent_id_obj.id,
            "instruction": None,
            "receiver": target_aid,
            "message": encoded,
            "timestamp": str(int(time.time() * 1000)),
        }
    }
    return session.message_client.send_msg(json.dumps(data))


class GroupClient:
    """Core request/response transport for ACP group protocol."""

    def __init__(self, agent_id_obj, session_id, target_aid):
        self._agent_id = agent_id_obj
        self._session_id = session_id
        self._target_aid = target_aid
        self._pending = {}  # request_id -> {event, response}
        self._seq = 0
        self._timeout = 30
        self._lock = threading.Lock()
        self._on_message_push = None  # callback(group_id, message_dict)

    def set_on_message_push(self, callback):
        """Set callback for incoming group message pushes: callback(group_id, message)"""
        self._on_message_push = callback

    def _next_id(self):
        self._seq += 1
        return f"{self._agent_id.id}-{int(time.time()*1000)}-{self._seq}"

    def send_request(self, action, group_id='', params=None):
        req_id = self._next_id()
        req = {'action': action, 'request_id': req_id}
        if group_id:
            req['group_id'] = group_id
        if params is not None:
            req['params'] = params

        event = threading.Event()
        with self._lock:
            self._pending[req_id] = {'event': event, 'response': None}

        payload = json.dumps(req)
        try:
            send_raw_to_session(self._agent_id, self._session_id, self._target_aid, payload)
        except Exception as e:
            with self._lock:
                self._pending.pop(req_id, None)
            raise

        if not event.wait(timeout=self._timeout):
            with self._lock:
                self._pending.pop(req_id, None)
            raise TimeoutError(f'Group request timeout: {action}')

        with self._lock:
            entry = self._pending.pop(req_id, None)
        resp = entry['response'] if entry else None
        if not resp:
            raise Exception(f'No response for {action}')
        if resp.get('code', -1) != 0:
            raise Exception(resp.get('error', f'{action} failed: code={resp.get("code")}'))
        return resp

    def handle_incoming(self, payload_str):
        """Try to handle as group protocol message. Returns True if handled."""
        try:
            data = json.loads(payload_str) if isinstance(payload_str, str) else payload_str
        except Exception:
            return False
        if not isinstance(data, dict):
            return False

        req_id = data.get('request_id', '')
        if req_id:
            with self._lock:
                entry = self._pending.get(req_id)
            if entry:
                entry['response'] = data
                entry['event'].set()
                return True

        action = data.get('action', '')
        event = data.get('event', '')
        if action in ('message_push', 'message_batch_push') or event:
            if self._on_message_push:
                try:
                    d = data.get('data', {})
                    if action == 'message_batch_push':
                        for msg in d.get('messages', []):
                            gid = msg.get('group_id', '') or d.get('group_id', '')
                            self._on_message_push(gid, msg)
                    else:
                        gid = d.get('group_id', '')
                        self._on_message_push(gid, d)
                except Exception as e:
                    log_warning(f'[GroupClient] message_push callback error: {e}')
            return True

        return False


class GroupOperations:
    """High-level group operations wrapping GroupClient."""

    def __init__(self, client: GroupClient, target_aid: str):
        self._client = client
        self._target = target_aid

    def _req(self, action, group_id='', params=None):
        return self._client.send_request(action, group_id, params)

    def register_online(self):
        self._req('register_online')

    def create_group(self, name, visibility='', description=''):
        params = {'name': name}
        if visibility:
            params['visibility'] = visibility
        if description:
            params['description'] = description
        resp = self._req('create_group', '', params)
        d = resp.get('data', {})
        return {'group_id': d.get('group_id', ''), 'group_url': d.get('group_url', '')}

    def send_message(self, group_id, content, content_type='text'):
        params = {'content': content, 'content_type': content_type}
        resp = self._req('send_message', group_id, params)
        d = resp.get('data', {})
        return {'msg_id': d.get('msg_id', 0), 'timestamp': d.get('timestamp', 0)}

    def pull_messages(self, group_id, after_msg_id=0, limit=0):
        params = {}
        if after_msg_id > 0:
            params['after_msg_id'] = after_msg_id
        if limit > 0:
            params['limit'] = limit
        resp = self._req('pull_messages', group_id, params or None)
        d = resp.get('data', {})
        return {
            'messages': d.get('messages', []),
            'has_more': d.get('has_more', False),
            'latest_msg_id': d.get('latest_msg_id', 0),
        }

    def get_group_info(self, group_id):
        resp = self._req('get_group_info', group_id)
        return resp.get('data', {})

    def get_members(self, group_id):
        resp = self._req('get_members', group_id)
        d = resp.get('data', {})
        return {'members': d.get('members', [])}

    def request_join(self, group_id, message=''):
        params = {'message': message} if message else None
        resp = self._req('request_join', group_id, params)
        d = resp.get('data', {})
        return {'status': d.get('status', 'pending'), 'request_id': d.get('request_id', '')}

    def use_invite_code(self, group_id, code):
        self._req('use_invite_code', group_id, {'code': code})

    def leave_group(self, group_id):
        self._req('leave_group', group_id)

    def create_invite_code(self, group_id, options=None):
        resp = self._req('create_invite_code', group_id, options)
        return resp.get('data', {})

    def list_my_groups(self, status=0):
        params = {'status': status} if status else None
        resp = self._req('list_my_groups', '', params)
        d = resp.get('data', {})
        return {'groups': d.get('groups', []), 'total': d.get('total', 0)}

    def review_join_request(self, group_id, agent_id, action, reason=''):
        params = {'agent_id': agent_id, 'action': action}
        if reason:
            params['reason'] = reason
        self._req('review_join_request', group_id, params)

    def get_pending_requests(self, group_id):
        resp = self._req('get_pending_requests', group_id)
        d = resp.get('data', {})
        return {'requests': d.get('requests', [])}

    def update_duty_config(self, group_id, config):
        self._req('update_duty_config', group_id, {'duty_config': config})

    def get_duty_status(self, group_id):
        resp = self._req('get_duty_status', group_id)
        d = resp.get('data', {})
        return {'config': d.get('config', {}), 'state': d.get('state', {})}

    @staticmethod
    def parse_group_url(group_url):
        from urllib.parse import urlparse
        parsed = urlparse(group_url)
        target_aid = parsed.hostname or ''
        group_id = parsed.path.lstrip('/')
        if not target_aid or not group_id:
            raise Exception(f'Invalid group URL: {group_url}')
        return target_aid, group_id



# -*- coding: utf-8 -*-
# Copyright 2025 AgentUnion Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import requests
import datetime
import socket
import threading
import time
from typing import Optional
from agentcp.base.log import log_debug, log_error, log_exception, log_info, log_warning
from agentcp.base.auth_client import AuthClient
from agentcp.utils import tls_config as tls

from agentcp.msg.message_serialize import *
from ..context import ErrorContext, exceptions


class HeartbeatClient:
    # 重连相关常量
    MAX_SEND_FAILURES = 3           # 发送失败触发重连的阈值
    MAX_RECV_FAILURES = 3           # 接收失败触发重连的阈值
    MAX_MISSED_HEARTBEATS = 3       # 心跳响应超时阈值（错过次数）
    RECONNECT_BACKOFF_MAX = 30      # 重连退避上限（秒）
    SOCKET_TIMEOUT = 1.0            # socket 超时时间（秒）

    def __init__(self, agent_id: str, server_url: str, aid_path: str, seed_password: str):
        self.agent_id = agent_id
        self.server_url = server_url
        self.seed_password = seed_password
        self.port = 0  # server_port
        self.sign_cookie = 0
        self.udp_socket = None
        self.local_ip = "0.0.0.0"
        self.local_port = 0
        self.server_ip = "127.0.0.1"
        self.heartbeat_interval = 5000
        self.is_running = False
        self.is_sending_heartbeat = False
        self.send_thread: Optional[threading.Thread] = None
        self.receive_thread: Optional[threading.Thread] = None
        self.msg_seq = 0
        self.last_hb = 0
        self.message_listener = None
        self.auth_client = AuthClient(agent_id, server_url, aid_path, seed_password)
        self.on_recv_invite = None

        # 新增：用于自动恢复的状态
        self._socket_lock = threading.Lock()        # 保护 socket 操作
        self._reconnect_lock = threading.Lock()     # 防止并发重连
        self._last_reconnect_ts = 0                 # 上次重连时间戳
        self._last_hb_recv = 0                      # 上次收到心跳响应的时间戳
        self._send_failures = 0                     # 连续发送失败次数
        self._recv_failures = 0                     # 连续接收失败次数

    def initialize(self):
        self.sign_in()

    def sign_in(self) -> bool:
        data = self.auth_client.sign_in()
        if data is None:
            log_error("sign_in failed: data is None")
            return False
        self.server_ip = data.get("server_ip")
        self.port = int(data.get("port", 0))
        self.sign_cookie = data.get("sign_cookie")
        log_info(f'signin {self.server_ip} {self.port} {self.sign_cookie}')

        return self.server_ip is not None and self.port != 0 and self.sign_cookie is not None

    def sign_out(self):
        self.auth_client.sign_out()

    def set_on_recv_invite(self, listener):
        """设置消息监听器"""
        self.on_recv_invite = listener

    # ========== 新增：Socket 生命周期管理 ==========

    def _create_socket(self):
        """创建并绑定 UDP socket，设置超时"""
        with self._socket_lock:
            self._close_socket_internal()
            self.udp_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            self.udp_socket.settimeout(self.SOCKET_TIMEOUT)
            self.udp_socket.bind((self.local_ip, 0))  # 使用新端口
            self.local_ip, self.local_port = self.udp_socket.getsockname()
            log_info(f"UDP socket created and bound to {self.local_ip}:{self.local_port}")

    def _close_socket_internal(self):
        """内部方法：关闭 socket（不加锁，由调用方保证线程安全）"""
        if self.udp_socket is not None:
            try:
                self.udp_socket.close()
            except Exception as e:
                log_warning(f"Close socket error: {e}")
            self.udp_socket = None

    def _close_socket(self):
        """安全关闭 socket"""
        with self._socket_lock:
            self._close_socket_internal()

    def _reconnect(self, reason: str):
        """重连：限流/退避后执行 sign_in() + _create_socket()"""
        if not self.is_running:
            log_debug(f"Reconnect skipped (client offline): {reason}")
            return False
        if not self._reconnect_lock.acquire(blocking=False):
            log_debug(f"Reconnect already in progress, skip: {reason}")
            return False

        try:
            now = time.time()
            # 限流：距离上次重连至少间隔 5 秒
            elapsed = now - self._last_reconnect_ts
            if elapsed < 5:
                backoff = min(5 - elapsed, self.RECONNECT_BACKOFF_MAX)
                log_info(f"Reconnect backoff: waiting {backoff:.1f}s")
                time.sleep(backoff)

            log_info(f"Reconnecting due to: {reason}")
            self._last_reconnect_ts = time.time()

            # 重新登录
            if not self.sign_in():
                log_error("Reconnect failed: sign_in returned False")
                return False

            # 重建 socket
            self._create_socket()

            # 重置失败计数
            self._send_failures = 0
            self._recv_failures = 0
            self._last_hb_recv = int(time.time() * 1000)

            log_info("Reconnect successful")
            return True
        except Exception as e:
            log_error(f"Reconnect exception: {e}")
            return False
        finally:
            self._reconnect_lock.release()

    # ========== 发送心跳（带异常恢复和超时检测） ==========

    def __send_heartbeat(self):
        backoff = 1  # 初始退避时间（秒）

        while self.is_sending_heartbeat and self.is_running:
            try:
                current_time_ms = int(datetime.datetime.now().timestamp() * 1000)

                # 检查心跳响应超时
                if self._last_hb_recv > 0:
                    timeout_threshold = self.MAX_MISSED_HEARTBEATS * self.heartbeat_interval
                    if current_time_ms - self._last_hb_recv > timeout_threshold:
                        log_warning(f"Heartbeat response timeout: {current_time_ms - self._last_hb_recv}ms > {timeout_threshold}ms")
                        self._reconnect("heartbeat_response_timeout")
                        backoff = 1
                        continue

                # 发送心跳
                if current_time_ms > (self.last_hb + self.heartbeat_interval):
                    log_debug(f'send heartbeat message to {self.server_ip}:{self.port}')
                    self.last_hb = current_time_ms
                    self.msg_seq = self.msg_seq + 1
                    req = HeartbeatMessageReq()
                    req.header.MessageMask = 0
                    req.header.MessageSeq = self.msg_seq
                    req.header.MessageType = 513
                    req.header.PayloadSize = 100
                    req.AgentId = self.agent_id
                    req.SignCookie = self.sign_cookie
                    buf = io.BytesIO()
                    req.serialize(buf)
                    data = buf.getvalue()

                    with self._socket_lock:
                        if self.udp_socket is not None:
                            self.udp_socket.sendto(data, (self.server_ip, self.port))
                        else:
                            raise Exception("UDP socket is None")

                    # 发送成功，重置失败计数和退避
                    self._send_failures = 0
                    backoff = 1

                time.sleep(1)

            except Exception as e:
                self._send_failures += 1
                log_error(f"Heartbeat send error (failures={self._send_failures}): {e}")
                ErrorContext.publish(exceptions.SDKError(f"Heartbeat send error: {e}"))

                # 达到阈值，触发重连
                if self._send_failures >= self.MAX_SEND_FAILURES:
                    log_warning(f"Send failures reached threshold ({self.MAX_SEND_FAILURES}), triggering reconnect")
                    self._reconnect("send_failures_threshold")
                    backoff = 1
                else:
                    # 指数退避
                    time.sleep(backoff)
                    backoff = min(backoff * 2, self.RECONNECT_BACKOFF_MAX)

    # ========== 接收消息（可中断、可恢复） ==========

    def _receive_messages(self):
        while self.is_running:
            try:
                # 使用 socket 超时，确保能定期检查 is_running
                with self._socket_lock:
                    if self.udp_socket is None:
                        time.sleep(0.5)
                        continue
                    sock = self.udp_socket

                try:
                    data, addr = sock.recvfrom(1536)
                except socket.timeout:
                    # 超时是正常的，继续循环检查 is_running
                    continue

                # 接收成功，重置失败计数
                self._recv_failures = 0

                udp_header, offset = UdpMessageHeader.deserialize(data, 0)

                if udp_header.MessageType == 258:
                    hb_resp, offset = HeartbeatMessageResp.deserialize(data, 0)
                    self.heartbeat_interval = hb_resp.NextBeat

                    # 更新最后收到心跳响应的时间
                    self._last_hb_recv = int(datetime.datetime.now().timestamp() * 1000)

                    # 服务器端身份验证失败(比如服务器发生了异常重启)，需要重新登录
                    if hb_resp.NextBeat == 401:
                        log_warning(f"Heartbeat failed: {hb_resp.NextBeat}, triggering reconnect")
                        ErrorContext.publish(exceptions.SDKError(f"401,心跳", code=0))
                        self._reconnect("401_auth_failed")
                        continue

                    if self.heartbeat_interval <= 5000:
                        self.heartbeat_interval = 5000

                elif udp_header.MessageType == 259:
                    invite_req, offset = InviteMessageReq.deserialize(data, 0)
                    if self.on_recv_invite is not None:
                        ErrorContext.publish(exceptions.SDKError(f"收到邀请，加入session: {invite_req}", code=0))
                        self.on_recv_invite(invite_req)

                    resp = InviteMessageResp()
                    self.msg_seq = self.msg_seq + 1
                    resp.header.MessageMask = 0
                    resp.header.MessageSeq = self.msg_seq
                    resp.header.MessageType = 516
                    resp.AgentId = self.agent_id
                    resp.InviterAgentId = invite_req.InviterAgentId
                    resp.SignCookie = self.sign_cookie
                    buf = io.BytesIO()
                    resp.serialize(buf)
                    resp_data = buf.getvalue()

                    with self._socket_lock:
                        if self.udp_socket is not None:
                            self.udp_socket.sendto(resp_data, (self.server_ip, self.port))

            except socket.timeout:
                # 超时是正常的，继续循环
                continue
            except Exception as e:
                if not self.is_running:
                    break

                self._recv_failures += 1
                log_error(f"Receive message exception (failures={self._recv_failures}): {e}")
                ErrorContext.publish(exceptions.SDKError(f"Receive message exception: {e}"))

                # 达到阈值，触发重连
                if self._recv_failures >= self.MAX_RECV_FAILURES:
                    log_warning(f"Recv failures reached threshold ({self.MAX_RECV_FAILURES}), triggering reconnect")
                    self._reconnect("recv_failures_threshold")
                else:
                    time.sleep(1.5)

    def online(self):
        """开始心跳"""
        if self.is_running:
            return

        # 使用统一的 socket 创建方法
        self._create_socket()

        # 初始化心跳响应时间
        self._last_hb_recv = int(time.time() * 1000)

        self.is_running = True
        self.is_sending_heartbeat = True

        self.send_thread = threading.Thread(target=self.__send_heartbeat, daemon=True)
        self.receive_thread = threading.Thread(target=self._receive_messages, daemon=True)

        self.send_thread.start()
        self.receive_thread.start()
        log_info("Successfully went online")

    def offline(self):
        """停止心跳"""
        log_info("Going offline...")

        # 1. 先设置标志位，通知线程退出
        self.is_running = False
        self.is_sending_heartbeat = False

        # 2. 关闭 socket（会使阻塞的 recvfrom 抛出异常）
        self._close_socket()

        # 3. 等待线程退出
        if self.send_thread is not None and self.send_thread.is_alive():
            self.send_thread.join(timeout=3)
            if self.send_thread.is_alive():
                log_warning("Send thread did not exit in time")

        if self.receive_thread is not None and self.receive_thread.is_alive():
            self.receive_thread.join(timeout=3)
            if self.receive_thread.is_alive():
                log_warning("Receive thread did not exit in time")

        self.send_thread = None
        self.receive_thread = None
        log_info("Successfully went offline")
    
    def get_online_status(self, aids):
        try:
            ep_url = self.server_url + "/query_online_state"
            data = {
                "agent_id": f"{self.agent_id}",
                "signature": self.auth_client.signature,
                "aids": aids
            }
            response = requests.post(ep_url, json=data, verify=tls.requests_verify(), proxies={}, timeout=(3, 10))
            if response.status_code == 200:
                log_info(f"get_online_status ok:{response.json()}")
                return response.json()["data"]
            else:
                log_error(f"get_online_status failed:{response.json()}")
                return []
        except Exception as e:
            log_exception(f"get_online_status exception: {e}")
            return []



# -*- coding: utf-8 -*-
# Copyright 2025 AgentUnion Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import websocket
import os
import ssl
from agentcp.msg.wss_binary_message import decode_wss_binary_buffer
from agentcp.utils import tls_config as tls
from agentcp.utils.proxy_bypass import ensure_no_proxy_for_local_env, without_proxy_env

ensure_no_proxy_for_local_env()
def download_file_via_websocket(wss_url, save_path):
    """
    通过WebSocket下载文件并保存到指定路径
    :param wss_url: WebSocket连接地址
    :param file_extension: 文件扩展名
    :param save_path: 文件保存路径
    """

    os.makedirs(os.path.dirname(save_path), exist_ok=True)
    file_stream_pulled_file = open(save_path, "wb")
    print(f"文件保存路径:{save_path}")
    def on_message(ws, message):
        try:
            if isinstance(message, bytes):
                bin_msg = decode_wss_binary_buffer(message)
                if bin_msg is None:
                    return
                if bin_msg.msg_type == 0x5 and bin_msg.content_type == 0x5:
                    print(f"收到文件流:offset={bin_msg.reserved}, size={bin_msg.payload_length}, crc32={bin_msg.crc32}")
                    if file_stream_pulled_file is not None:
                        file_stream_pulled_file.write(bin_msg.payload)
                else:
                    txt_msg = bin_msg.payload.decode()
                    if txt_msg is not None and len(txt_msg) > 0:
                        print(f"File_Stream pull handler 收到消息收到二进制消息: {txt_msg}")
                        on_message(ws, txt_msg)
            elif isinstance(message, str):
                import json
                js = json.loads(message)
                if "cmd" not in js or "data" not in js:
                    return
                cmd = js["cmd"]
                data = js["data"]
                if cmd == "close_file_stream_req":
                    if file_stream_pulled_file is not None:
                        file_stream_pulled_file.close()
                        file_stream_pulled_file = None
                        ws.close()

        except Exception as e:
            print(f"file_stream_pull_handler在处理收到的消息时发生异常 {str(e)}")

    def on_error(ws, error):
        print(f"发生错误: {error}")
        ws.close()  # 发生错误时主动关闭连接

    def on_close(ws, close_status_code, close_msg):
        print("连接已关闭")
        ws = None  # 确保引用被清除

    def on_open(ws):
        print("连接已建立，等待文件数据...")

    # 创建WebSocket连接，禁用SSL验证
    ws = websocket.WebSocketApp(wss_url,
                              on_open=on_open,
                              on_message=on_message,
                              on_error=on_error,
                              on_close=on_close)
    
    # 运行时不验证SSL证书
    with without_proxy_env(enabled=True):
        ws.run_forever(
            sslopt=tls.websocket_sslopt(),
        )



# -*- coding: utf-8 -*-
# Copyright 2025 AgentUnion Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import sqlite3
import os
import threading
import json
from agentcp.base.log import log_debug, log_error, log_exception, log_info
class DBManager:
    
    def __init__(self,aid_path,aid):
        db_path = os.path.join(aid_path,"data")
        os.path.exists(db_path) or os.makedirs(db_path)
        self._db_path = os.path.join(db_path, "agentid_data.db")
        self.aid = aid
        self._local = threading.local()
        self._closed = False

        # 初始化时开启 WAL 模式（只需执行一次，持久化到数据库文件）
        conn = sqlite3.connect(self._db_path)
        result = conn.execute("PRAGMA journal_mode=WAL").fetchone()
        if result is None or result[0].lower() != "wal":
            log_info(f"WAL 模式设置失败，当前模式: {result[0] if result else 'unknown'}，并发写入性能可能受限")
        conn.close()

        self.create_table()

    def _get_conn(self) -> sqlite3.Connection:
        if self._closed:
            log_error("DBManager 已关闭，不应再获取连接")
        conn = getattr(self._local, 'conn', None)
        if conn is None:
            conn = sqlite3.connect(self._db_path, timeout=30)
            self._local.conn = conn
        return conn

    def close(self):
        """关闭当前线程的连接，每个线程应在退出前调用"""
        conn = getattr(self._local, 'conn', None)
        if conn is not None:
            try:
                conn.close()
            except Exception:
                pass
            self._local.conn = None

    def shutdown(self):
        """标记关闭并清理当前线程连接。各工作线程应在退出前自行调用 close()"""
        self._closed = True
        self.close()

    def add_friend_agent(self,friend_aid,name,avaUrl,description):
        try:
            friend_table = f"friend"
            conn = self._get_conn()
            cursor = conn.cursor()
            # 使用INSERT OR REPLACE来实现upsert功能
            # 如果aid存在就更新，不存在就插入
            cursor.execute(f'''INSERT OR REPLACE INTO {friend_table} (aid, name, avaurl, description) VALUES (?, ?, ?, ?)''',
                            (friend_aid, name, avaUrl, description))
            conn.commit()
            result = cursor.rowcount > 0
            cursor.close()  # 关闭cursor对象
            return result
        except sqlite3.Error as e:
            log_error(f"add_friend_agent数据库操作失败: {str(e)}")
            return False

    def set_friend_agent(self,friend_aid,name):
        try:
            friend_table = f"friend"
            conn = self._get_conn()
            cursor = conn.cursor()

            # 首先检查aid是否存在
            cursor.execute(f'''SELECT avaurl, description FROM {friend_table} WHERE aid = ?''', (friend_aid,))
            existing_record = cursor.fetchone()

            if existing_record:
                # 如果存在，只更新name，保持avaurl和description不变
                cursor.execute(f'''UPDATE {friend_table} SET name = ? WHERE aid = ?''',
                            (name, friend_aid))
            else:
                # 如果不存在，插入新记录，avaurl和description设为空或默认值
                cursor.execute(f'''INSERT INTO {friend_table} (aid, name, avaurl, description) VALUES (?, ?, ?, ?)''',
                            (friend_aid, name, '', ''))

            conn.commit()
            result = cursor.rowcount > 0
            cursor.close()  # 关闭cursor对象
            return result
        except sqlite3.Error as e:
            log_error(f"set_friend_agent数据库操作失败: {str(e)}")
            return False

    def delete_friend_agent(self,friend_aid):
        try:
            friend_table = f"friend"
            # 修改为使用动态生成的friend_table表名
            conn = self._get_conn()
            cursor = conn.cursor()
            cursor.execute(f'''DELETE FROM {friend_table} WHERE aid = ?''', (friend_aid,))
            conn.commit()
            result = cursor.rowcount > 0
            cursor.close()  # 关闭cursor对象
            return result
        except sqlite3.Error as e:
            log_error(f"delete_friend_agent数据库操作失败: {str(e)}")
            return False
    
    def delete_session(self,session_id):
        try:
            conversation_table = f"conversation"
            messages_table = f"messages"
            chat_config_table = f"chat_config"
            # 修改为使用动态生成的friend_table表名
            conn = self._get_conn()
            cursor = conn.cursor()
            cursor.execute(f'''DELETE FROM {conversation_table} WHERE session_id = ?''', (str(session_id),))
            conn.commit()
            cursor.execute(f'''DELETE FROM {messages_table} WHERE session_id =?''', (session_id,))
            conn.commit()
            cursor.execute(f'''DELETE FROM {chat_config_table} WHERE session_id =?''', (session_id,))
            conn.commit()
            result = cursor.rowcount > 0
            cursor.close()  # 关闭cursor对象
            return result
        except sqlite3.Error as e:
            log_error(f"delete_session数据库操作失败: {str(e)}")
            return False

    def get_friend_agent_list(self):
        try:
            friend_table = f"friend"
            # 修改为使用动态生成的friend_table表名
            cursor = self._get_conn().cursor()
            cursor.execute(f'''SELECT * FROM {friend_table}''')
            #columns = [column[0] for column in cursor.description]  # 获取列名
            rows = cursor.fetchall()
            result = []
            for row in rows:
                row_dict = {
                    "id":row[0],
                    "aid":row[1],
                    "name":row[2],
                    "avaurl":row[3],
                    "description":row[4]
                }
                result.append(row_dict)
            cursor.close()
            return result
        except sqlite3.Error as e:
            log_error(f"get_friend_agent_list数据库操作失败: {str(e)}")
            return []
        
    
    def _create_table(self):
        # 身份表，TODO 需考虑加密..
        # cursor.execute("""DELETE FROM agentids""")
        conn = self._get_conn()
        cursor = conn.cursor()
        cursor.execute('''CREATE TABLE IF NOT EXISTS agentids (
            aid TEXT PRIMARY KEY,
            ep_aid TEXT,
            ep_url TEXT,
            avaUrl TEXT,
            name TEXT,
            description TEXT)
        ''')

        conn.commit()
        cursor.close()
    
    def create_table(self):
        self._create_table()
        conn = self._get_conn()
        cursor = conn.cursor()
        conversation_table = f"conversation"
        messages_table = f"messages"
        friend_table = f"friend"
        chat_config_table = f"chat_config"
        # cursor.execute(f"DROP TABLE IF EXISTS {conversation_table}")

        cursor.execute(f'''CREATE TABLE IF NOT EXISTS {conversation_table} (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            session_id TEXT NOT NULL UNIQUE,
            identifying_code TEXT NOT NULL,
            main_aid TEXT NOT NULL,
            name TEXT NOT NULL,
            type TEXT NOT NULL,
            timestamp DATETIME DEFAULT (strftime('%Y-%m-%d %H:%M:%S', 'now', 'localtime'))
        )''')
        
        #cursor.execute(f"DROP TABLE IF EXISTS {messages_table}")
        
        cursor.execute(f'''CREATE TABLE IF NOT EXISTS {messages_table} (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            message_id TEXT,  -- 消息id, 用于去重
            session_id TEXT,  -- 会话id
            role TEXT NOT NULL,  -- 消息发送者,
            message_aid TEXT NOT NULL, 
            parent_message_id INTEGER,  -- 关联到父消息的id
            to_aids TEXT NOT NULL,  -- 消息发送者,
            content TEXT NOT NULL,  -- 消息内容,
            instruction Text,  -- 指令,
            type TEXT NOT NULL,  -- 消息状态，如"sent", "received",
            status TEXT NOT NULL,  -- 消息状态，如"error", "success",
            timestamp DATETIME DEFAULT (strftime('%Y-%m-%d %H:%M:%S', 'now', 'localtime'))
        )''')
        
        cursor.execute(f'''CREATE TABLE IF NOT EXISTS {chat_config_table} (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            session_id INTEGER NOT NULL,  
            aid TEXT NOT NULL,  
            avaurl TEXT,
            description TEXT, 
            post_data TEXT
        )''')
        
        cursor.execute(f'''CREATE TABLE IF NOT EXISTS {friend_table} (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            aid TEXT NOT NULL,
            name TEXT,
            avaurl TEXT,
            description TEXT
        )''')


        cursor.execute(f'''CREATE TABLE IF NOT EXISTS session_server (
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                session_id TEXT NOT NULL,
                session_server TEXT
        )''')
        
        conn.commit()
        cursor.close()

    def insert_session_server(self, session_id, session_server):
        try:
            conn = self._get_conn()
            cursor = conn.cursor()
            cursor.execute('''INSERT INTO session_server (session_id, session_server)
                            VALUES (?, ?)''',
                            (session_id, session_server))
            conn.commit()
            result = cursor.lastrowid
            cursor.close()
            return result
        except sqlite3.Error as e:
            log_error(f"insert_session_server数据库操作失败: {str(e)}")
            return None

    def get_session_server(self, session_id):
        try:
            cursor = self._get_conn().cursor()
            cursor.execute('''SELECT session_server FROM session_server 
                            WHERE session_id = ?''', 
                            (session_id,))
            result = cursor.fetchone()
            cursor.close()
            return result[0] if result else None
        except sqlite3.Error as e:
            log_error(f"get_session_server数据库操作失败: {str(e)}")
            return None
        
    def load_session_history(self, session_id):
        try:
            cursor = self._get_conn().cursor()
            sesion_table = f"conversation"
            cursor.execute(f'''SELECT * FROM {sesion_table} WHERE session_id =?''', (session_id,))
            columns = ['id','session_id','identifying_code','main_aid','name','type','timestamp']
            results = [dict(zip(columns, row)) for row in cursor.fetchall()]
            cursor.close()
            return results
        except sqlite3.Error as e:
            log_error(f"load_session_history数据库操作失败: {str(e)}")
            return None
    
    
    def update_aid_info(self, aid, avaUrl, name, description):
        try:
            conn = self._get_conn()
            cursor = conn.cursor()
            cursor.execute(
                '''UPDATE agentids SET avaUrl =?, name =?, description =? WHERE aid =?''',
                (avaUrl, name, description, aid),
            )
            conn.commit()
            result = cursor.rowcount > 0
            cursor.close()  # 关闭cursor对象
            return result
        except sqlite3.Error as e:
            log_error(f"update_aid_info数据库操作失败: {str(e)}")
            return False
    
    def create_session(self, aid,session_id,identifying_code,name, type):
        try:
            conn = self._get_conn()
            cursor = conn.cursor()
            conversation_table = f"conversation"
            cursor.execute(
                f'''INSERT OR IGNORE INTO {conversation_table} (session_id,identifying_code,main_aid,name,type) VALUES (?,?,?,?,?)''',
                (session_id, identifying_code, aid, name, type),
            )
            conn.commit()
            result = cursor.lastrowid
            cursor.close()  # 关闭cursor对象
            return result
        except sqlite3.Error as e:
            log_error(f"create_conversation数据库操作失败: {str(e)}")
            return None

    def invite_member(self, aid, session_id, invite_aid:str):
        try:
            conn = self._get_conn()
            cursor = conn.cursor()
            chat_config_table = f"chat_config"
            cursor.execute(
                f'''INSERT INTO {chat_config_table} (session_id,aid,avaurl,description, post_data) VALUES (?,?,?,?,?)''',
                (session_id, invite_aid, "", "", ""),
            )
            conn.commit()
            result = cursor.lastrowid
            cursor.close()  # 关闭cursor对象
            return result
        except sqlite3.Error as e:
            log_error(f"invite_member数据库操作失败: {str(e)}")
            return False
        except Exception as e:
            log_error(f"invite_member数据库操作失败: {str(e)}")
            return False
    
    def get_message_by_id(self, aid, session_id,message_id):
        try:
            cursor = self._get_conn().cursor()
            messages_table = f"messages"
            cursor.execute(
                f'''SELECT * FROM {messages_table} WHERE session_id =? AND message_id =?''',
                (session_id, message_id),
            )
            columns = [
                'id',
                'message_id',
                'session_id',
                'role',
                'message_aid',
                'parent_message_id',
                'to_aids',
                'content',
                'instruction',
                'type',
                'status',
                'timestamp',
            ]
            results = [dict(zip(columns, row)) for row in cursor.fetchall()]
            cursor.close()  # 关闭cursor对象
            return results
        except sqlite3.Error as e:
            log_error(f"get_message_by_id数据库操作失败: {str(e)}")
            return []

    def update_message(self, message):
        try:
            conn = self._get_conn()
            cursor = conn.cursor()
            messages_table = f"messages"
            content_str = message.get("content")
            if content_str is not None:
                if isinstance(content_str, (dict, list)):
                    content_str = json.dumps(content_str)
                else:
                    content_str = str(content_str)
            cursor.execute(
                f'''UPDATE {messages_table} SET content =?,status =? WHERE id =?''',
                (content_str, message.get("status"), message.get("id")),
            )
            conn.commit()
            result = cursor.rowcount > 0
            cursor.close()  # 关闭cursor对象
            return result
        except sqlite3.Error as e:
            log_error(f"update_message数据库操作失败: {str(e)}")
            return False

    def insert_message(self, role,aid,conversation_id, message_aid, parent_message_id, to_aids, instruction,content, type, status,message_id=''):
        try:
            conn = self._get_conn()
            cursor = conn.cursor()
            messages_table = f"messages"
            cursor.execute(
                f'''INSERT INTO {messages_table} (session_id, role,message_id,message_aid, parent_message_id, to_aids, instruction,content, type, status) VALUES (?,?,?,?,?,?,?,?,?,?)''',
                (conversation_id, role, message_id, message_aid, parent_message_id, to_aids, instruction, content, type, status),
            )
            conn.commit()
            result = cursor.lastrowid
            cursor.close()  # 关闭cursor对象
            return result
        except sqlite3.Error as e:
            log_error(f"insert_message数据库操作失败: {str(e)}")
            return None

    def load_aid(self, aid):
        try:
            cursor = self._get_conn().cursor()
            cursor.execute('''SELECT ep_aid, ep_url,avaUrl,name,description FROM agentids WHERE aid = ?''', (aid,))
            result = cursor.fetchone()
            cursor.close()  # 关闭cursor对象
            if result:
                return result[0], result[1],result[2],result[3],result[4]
            else:
                return None, None,None,None,None
        except sqlite3.Error as e:
            log_error(f"load_aid数据库操作失败: {str(e)}")
            return None, None,None,None,None

    def create_aid(self, aid,ep_aid = "",ep_url = "",avaUrl = "",name = "",description=""):
        try:
            conn = self._get_conn()
            cursor = conn.cursor()
            # 将私钥和CSR序列化为PEM格式字符串
            cursor.execute('''INSERT INTO agentids (aid, ep_aid, ep_url,avaUrl,name,description)
                                VALUES (?,?,?,?,?,?)''',
                                (aid, ep_aid , ep_url,avaUrl,name,description))
            conn.commit()
            cursor.close()  # 关闭cursor对象
            return ""
        except sqlite3.Error as e:
            log_error(f"save_aid数据库操作失败: {str(e)}")
            raise RuntimeError(f"save_aid数据库操作失败: {str(e)}")
            
    def update_aid(self, aid, ep_aid, ep_url):
        try:
            conn = self._get_conn()
            cursor = conn.cursor()
            cursor.execute('''UPDATE agentids SET ep_aid =?, ep_url =? WHERE aid =?''', (ep_aid, ep_url, aid))
            conn.commit()
            result = cursor.rowcount > 0
            cursor.close()  # 关闭cursor对象
            return result
        except sqlite3.Error as e:
            log_error(f"update_aid数据库操作失败: {str(e)}")
            return False
    
    def get_agentid_list(self):
        try:
            cursor = self._get_conn().cursor()
            cursor.execute('''SELECT aid FROM agentids''')
            result = [row[0] for row in cursor.fetchall()]  # 提取每个元组的第一个元素
            cursor.close()  # 关闭cursor对象
            return result
        except sqlite3.Error as e:
            log_error(f"get_agentid_list数据库操作失败: {str(e)}")
            return []

    def get_conversation_by_id(self, aid, conversation_id):
        try:
            cursor = self._get_conn().cursor()
            conversation_table = f"conversation"
            cursor.execute(f'''SELECT id, session_id,identifying_code,main_aid,name,type FROM {conversation_table} WHERE session_id =?''', (conversation_id,))
            result = cursor.fetchone()
            cursor.close()  # 关闭cursor对象
            if result:
                return {
                    'id': result[0],
                    'session_id': result[1],
                    'identifying_code': result[2],
                    'main_aid': result[3],
                    'name': result[4],
                    'type': result[5]
                }
            else:
                return None
        except sqlite3.Error as e:
            log_error(f"get_conversation_by_id数据库操作失败: {str(e)}")
            return None
          
    def get_conversation_list(self, aid, main_aid, page=1, page_size=10):
        try:
            offset = (page - 1) * page_size
            cursor = self._get_conn().cursor()
            conversation_table = f"conversation"
            if main_aid == "" or main_aid == None:
                cursor.execute(
                    f'''SELECT id, session_id, identifying_code, name, type, timestamp
                        FROM {conversation_table}
                        ORDER BY timestamp DESC
                        LIMIT? OFFSET?''',  
                    (page_size, offset)
                )
            else:
                cursor.execute(
                    f'''SELECT id, session_id, identifying_code, name, type, timestamp 
                        FROM {conversation_table} 
                        WHERE main_aid = ? 
                        ORDER BY timestamp DESC 
                        LIMIT ? OFFSET ?''', 
                    (main_aid, page_size, offset))
            
            # 将查询结果转换为字典列表
            columns = ['id', 'session_id', 'identifying_code', 'name', 'type', 'timestamp']
            results = [dict(zip(columns, row)) for row in cursor.fetchall()]
            cursor.close()  # 关闭cursor对象
            # 返回JSON格式字符串
            return results
            
        except sqlite3.Error as e:
            log_error(f"get_conversation_list数据库操作失败: {str(e)}")
            return []
    
    def get_conversation_messages(self, conversation_id):
        try:
            cursor = self._get_conn().cursor()
            cursor.execute('''SELECT id, aid, content, timestamp FROM messages WHERE session_id =? ORDER BY timestamp ASC''', (conversation_id,))
            result = cursor.fetchall()
            cursor.close()  # 关闭cursor对象
            return result
        except sqlite3.Error as e:
            log_error(f"get_conversation_messages数据库操作失败: {str(e)}")
            return []
    
    def get_conversation_config(self, conversation_id):
        try:
            cursor = self._get_conn().cursor()
            cursor.execute('''SELECT id, aid, avaurl, description, post_data FROM chat_config WHERE session_id =?''', (conversation_id,))
            result = cursor.fetchall()
            cursor.close()  # 关闭cursor对象
            return result
        except sqlite3.Error as e:
            log_error(f"get_conversation_config数据库操作失败: {str(e)}")
            return []

    def add_conversation_config(self, conversation_id, aid, avaurl, description, post_data):
        try:
            conn = self._get_conn()
            cursor = conn.cursor()
            cursor.execute('''INSERT INTO chat_config (conversation_id, aid, avaurl, description, post_data) VALUES (?,?,?,?,?)''', (conversation_id, aid, avaurl, description, post_data))
            conn.commit()
            result = cursor.lastrowid
            cursor.close()  # 关闭cursor对象
            return result
        except sqlite3.Error as e:
            log_error(f"add_conversation_config数据库操作失败: {str(e)}")
            return None

    def update_conversation_config(self, config_id, avaurl, description, post_data):
        try:
            conn = self._get_conn()
            cursor = conn.cursor()
            cursor.execute('''UPDATE chat_config SET avaurl = ?, description = ?, post_data = ? WHERE id = ?''', (avaurl, description, post_data, config_id))
            conn.commit()
            result = cursor.rowcount > 0
            cursor.close()  # 关闭cursor对象
            return result
        except sqlite3.Error as e:
            log_error(f"update_conversation_config数据库操作失败: {str(e)}")
            return False
    
    def get_message_list(self, aid, session_id, page=1, page_size=10):
        try:
            page_size = 100
            offset = (page - 1) * page_size
            cursor = self._get_conn().cursor()
            messages_table = f"messages"
            
            # 修正SQL语句，移除多余的括号
            cursor.execute(
                f'''SELECT id, session_id,message_id,role,message_aid, parent_message_id, to_aids,instruction, content, type, status, timestamp
                    FROM {messages_table}
                    WHERE session_id = ? 
                    ORDER BY timestamp ASC LIMIT ? OFFSET ?''', 
                (session_id, page_size, offset))
            
            # 将查询结果转换为字典列表
            columns = ['id','session_id','message_id','role','message_aid', 'parent_message_id', 'to_aids', 'instruction','content', 'type', 'status', 'timestamp']
            results = [dict(zip(columns, row)) for row in cursor.fetchall()]
            cursor.close()  # 关闭cursor对象
            # 返回JSON格式字符串
            return results
        except sqlite3.Error as e:
            log_error(f"get_message_list数据库操作失败: {str(e)}")
            return []
        # return [dict(row) for row in cursor.fetchall(
    
    def get_session_member_list(self,session_id):
        try:
            cursor = self._get_conn().cursor()
            chat_config_table = f"chat_config"
            cursor.execute(f'''SELECT * FROM {chat_config_table} WHERE session_id =?''', (session_id,))
            columns = [column[0] for column in cursor.description]  # 获取列名
            rows = cursor.fetchall()
            result = []
            for row in rows:
                if len(row) != len(columns):  # 添加长度校验
                    continue
                row_dict = {columns[i]: row[i] for i in range(len(columns))}
                result.append(row_dict)
            cursor.close()
            return result
        except sqlite3.Error as e:
            log_error(f"get_session_member_list数据库操作失败: {str(e)}")
            return []
    



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
acp-py 默认启动脚本
使用方式: python -m agentcp.start [args]
"""
import os
import time
import argparse
import json


def _create_session_with_target(aid, target_aid, session_name=None):
    """创建会话并邀请目标"""
    name = session_name or f"session_{int(time.time())}"
    session_id = aid.create_session(name, "ACP 会话")
    if session_id:
        aid.invite_member(session_id, target_aid)
    return session_id


def main():
    parser = argparse.ArgumentParser(description='启动 Python Agent')
    parser.add_argument('--data', '-d', default='./acp-data', help='数据目录 (默认: ./acp-data)')
    parser.add_argument('--password', '-p', default='agentcp@2025', help='种子密码 (默认: agentcp@2025，也可通过环境变量 ACP_SEED_PASSWORD 设置)')
    parser.add_argument('--name', '-n', default=None, help='Agent 名称 (默认: 自动生成)')
    parser.add_argument('--ap', default='aid.pub', help='AP 地址 (默认: aid.pub)')
    parser.add_argument('--debug', action='store_true', help='启用调试模式')
    parser.add_argument('--target', '-t', default=None, help='目标 Agent AID (用于发送消息)')
    parser.add_argument('--web', '-w', action='store_true', default=True, help='启动 Web 管理界面 (默认启用)')
    parser.add_argument('--cli', action='store_true', help='使用 CLI 交互模式 (不启动 Web)')
    parser.add_argument('--port', type=int, default=9528, help='Web 服务端口 (默认: 9528)')
    parser.add_argument('--no-browser', action='store_true', help='不自动打开浏览器')
    args = parser.parse_args()

    # 解析种子密码：环境变量 > CLI参数（含默认值）
    env_password = os.environ.get('ACP_SEED_PASSWORD')
    if env_password:
        args.password = env_password

    # Web 模式（默认）
    if not args.cli:
        from agentcp.web_server import start_server
        start_server(port=args.port, data_dir=args.data, seed_password=args.password, open_browser=not args.no_browser)
        return

    # 延迟导入，确保 PYTHONPATH 已设置
    from agentcp import AgentCP

    # 创建数据目录
    data_path = os.path.abspath(args.data)
    os.makedirs(data_path, exist_ok=True)

    print("=" * 50)
    print("ACP Python Agent")
    print("=" * 50)
    print(f"数据目录: {data_path}")

    # 创建 AgentCP 实例
    acp = AgentCP(
        data_path,
        seed_password=args.password,
        debug=args.debug,
        run_proxy=False
    )

    # 创建 Agent 的 AID
    agent_name = args.name or f"acp_py_{int(time.time())}"
    aid = acp.create_aid(ap=args.ap, agent_name=agent_name)
    print(f"Agent 名称: {agent_name}")
    print(f"Agent AID: {aid.id}")

    # 注册消息处理器（异步函数）
    async def handle_message(data):
        """处理收到的消息"""
        try:
            sender = data.get('sender', '未知')
            session_id = data.get('session_id', '')
            message_id = data.get('message_id', '')

            # 解析消息内容
            message_raw = data.get('message', '{}')
            message = json.loads(message_raw) if isinstance(message_raw, str) else message_raw

            # 处理消息列表
            if isinstance(message, list):
                message = message[0] if message else {}

            # 提取内容
            if isinstance(message, dict):
                content = message.get('content', message.get('text', str(message)))
                msg_type = message.get('type', '')
            else:
                content = str(message)
                msg_type = ''

            print(f"\n{'=' * 40}")
            print(f"[收到消息]")
            print(f"  发送者: {sender}")
            print(f"  内容: {content}")
            if msg_type:
                print(f"  类型: {msg_type}")
            if args.debug:
                print(f"  会话ID: {session_id}")
                print(f"  消息ID: {message_id}")
            print(f"{'=' * 40}")
            print(">>> ", end="", flush=True)

        except Exception as e:
            print(f"\n[消息处理错误] {e}")
            if args.debug:
                import traceback
                traceback.print_exc()
            print(">>> ", end="", flush=True)

    aid.add_message_handler(handle_message)

    # 上线
    aid.online()
    print("Agent 已上线")

    # 会话管理
    session_id = None
    target_aid = args.target

    # 如果指定了目标，创建会话
    if target_aid:
        print(f"目标 Agent: {target_aid}")
        session_id = _create_session_with_target(aid, target_aid, "acp_session")
        if session_id:
            print(f"会话ID: {session_id}")
            print(f"已邀请 {target_aid} 加入会话")

    print("=" * 50)
    print("命令:")
    print("  /target <aid>  - 设置目标 Agent")
    print("  /session       - 显示当前会话信息")
    print("  /aid           - 显示本机 AID")
    print("  /quit          - 退出")
    print("  其他输入       - 发送消息")
    print("=" * 50)

    # 交互式输入
    while True:
        try:
            msg = input(">>> ").strip()
            if not msg:
                continue

            # 处理命令
            if msg.startswith('/'):
                parts = msg.split(maxsplit=1)
                cmd = parts[0].lower()

                if cmd in ('/quit', '/exit', '/q'):
                    print("正在退出...")
                    break

                elif cmd == '/target':
                    if len(parts) > 1:
                        target_aid = parts[1].strip()
                        session_id = _create_session_with_target(aid, target_aid)
                        if session_id:
                            print(f"已设置目标: {target_aid}")
                            print(f"新会话ID: {session_id}")
                        else:
                            print("创建会话失败")
                    else:
                        print(f"当前目标: {target_aid}" if target_aid else "未设置目标，使用: /target <aid>")

                elif cmd == '/session':
                    print(f"会话ID: {session_id or '无'}")
                    print(f"目标AID: {target_aid or '无'}")

                elif cmd == '/aid':
                    print(f"本机 AID: {aid.id}")

                elif cmd == '/help':
                    print("命令:")
                    print("  /target <aid>  - 设置目标 Agent")
                    print("  /session       - 显示当前会话信息")
                    print("  /aid           - 显示本机 AID")
                    print("  /quit          - 退出")

                else:
                    print(f"未知命令: {cmd}，输入 /help 查看帮助")

            else:
                # 发送消息
                if not target_aid:
                    print("请先设置目标: /target <aid>")
                    continue

                if not session_id:
                    session_id = _create_session_with_target(aid, target_aid)
                    if not session_id:
                        print("创建会话失败")
                        continue

                aid.send_message(session_id, [target_aid], msg)
                print(f"[已发送] {msg}")

        except KeyboardInterrupt:
            print("\n正在退出...")
            break
        except EOFError:
            break

    # 清理资源
    try:
        aid.offline()
    except (Exception, KeyboardInterrupt):
        pass


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2025 AgentUnion Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import logging

from agentcp.base.env import Environ


def get_logger(name=__name__, level=Environ.LOG_LEVEL.get(logging.INFO)) -> logging.Logger:
    """
    Set up the log for the agentid module.
    """
    log = logging.getLogger(name)
    log.setLevel(level)
    if not log.handlers:
        console_handler = logging.StreamHandler()
        console_handler.setLevel(level)
        formatter = logging.Formatter(
            "%(asctime)s - %(name)s - %(filename)s:%(lineno)d - %(levelname)s - %(message)s"
        )
        console_handler.setFormatter(formatter)
        log.addHandler(console_handler)
    else:
        for handler in log.handlers:
            try:
                handler.setLevel(level)
            except Exception:
                pass
    return log


logger = None
log_enabled = True


def _ensure_logger() -> logging.Logger:
    """Ensure module-level logger is initialized to avoid None dereferences."""
    global logger
    if logger is None:
        try:
            logger = get_logger(name="agentid", level=Environ.LOG_LEVEL.get(logging.INFO))
        except Exception:
            logger = logging.getLogger("agentid")
    return logger


def set_log_enabled(enabled: bool, level: int):
    global log_enabled, logger
    log_enabled = enabled
    logger = get_logger(name="agentid", level=Environ.LOG_LEVEL.get(level))
    
def log_exception(e):
    global log_enabled
    if log_enabled:
        _ensure_logger().exception(e)
        
def log_info(content:str):
    global log_enabled
    if log_enabled and _ensure_logger().isEnabledFor(logging.INFO):
        _ensure_logger().info(content)
        
def log_error(content:str):
    global log_enabled
    if log_enabled and _ensure_logger().isEnabledFor(logging.ERROR):
        _ensure_logger().error(content)

def log_debug(content:str):
    global log_enabled
    if log_enabled and _ensure_logger().isEnabledFor(logging.DEBUG):
        _ensure_logger().debug(content)
        
def log_warning(content:str):
    global log_enabled
    if log_enabled and _ensure_logger().isEnabledFor(logging.WARNING):
        _ensure_logger().warning(content)

# 新增关键日志级别
def log_critical(content:str):
    global log_enabled
    if log_enabled and _ensure_logger().isEnabledFor(logging.CRITICAL):
        _ensure_logger().critical(content)

# 新增详细日志级别        
def log_verbose(content:str):
    global log_enabled
    if log_enabled and _ensure_logger().isEnabledFor(logging.DEBUG - 1):
        _ensure_logger().log(logging.DEBUG - 1, content)



# -*- coding: utf-8 -*-
"""
✅ P1-3新增: 消息处理指标收集器

提供详细的消息处理统计和监控指标
"""

import threading
import time
from typing import Dict, Any, List


class MessageMetrics:
    """消息处理指标收集器

    收集以下指标：
    - 消息接收总数
    - 派发成功/失败数
    - Handler 成功/失败数
    - 派发队列大小
    - 平均延迟
    - 延迟直方图（p50, p95, p99）
    """

    def __init__(self):
        self.lock = threading.Lock()

        # 基础计数器
        self.received_total = 0
        self.dispatched_success = 0
        self.dispatched_failed = 0
        self.handler_success = 0
        self.handler_failed = 0
        self.dispatch_queue_size = 0  # ✅ 派发队列大小

        # 延迟统计
        self.dispatch_latencies: List[float] = []  # 毫秒
        self.handler_latencies: List[float] = []  # 毫秒
        self.max_latency_samples = 1000  # 保留最近 1000 个样本

        # 平均值缓存
        self._avg_dispatch_latency_ms = 0.0
        self._avg_handler_latency_ms = 0.0

        # 启动时间
        self.start_time = time.time()

    def record_received(self):
        """记录收到消息"""
        with self.lock:
            self.received_total += 1

    def record_dispatch_success(self, latency_ms: float):
        """记录派发成功

        Args:
            latency_ms: 派发延迟（毫秒）
        """
        with self.lock:
            self.dispatched_success += 1
            self.dispatch_latencies.append(latency_ms)

            # 保持样本数量在限制内
            if len(self.dispatch_latencies) > self.max_latency_samples:
                self.dispatch_latencies.pop(0)

            # 更新平均值
            self._avg_dispatch_latency_ms = sum(self.dispatch_latencies) / len(self.dispatch_latencies)

    def record_dispatch_failure(self):
        """记录派发失败"""
        with self.lock:
            self.dispatched_failed += 1

    def record_handler_success(self, latency_ms: float):
        """记录 Handler 成功

        Args:
            latency_ms: 处理延迟（毫秒）
        """
        with self.lock:
            self.handler_success += 1
            self.handler_latencies.append(latency_ms)

            # 保持样本数量在限制内
            if len(self.handler_latencies) > self.max_latency_samples:
                self.handler_latencies.pop(0)

            # 更新平均值
            self._avg_handler_latency_ms = sum(self.handler_latencies) / len(self.handler_latencies)

    def record_handler_failure(self):
        """记录 Handler 失败"""
        with self.lock:
            self.handler_failed += 1

    def update_dispatch_queue_size(self, count: int):
        """更新派发队列计数

        Args:
            count: 当前派发队列大小
        """
        with self.lock:
            self.dispatch_queue_size = count

    def get_summary(self) -> Dict[str, Any]:
        """获取指标摘要

        Returns:
            包含所有指标的字典
        """
        with self.lock:
            # 计算成功率
            dispatch_rate = 0.0
            if self.received_total > 0:
                dispatch_rate = (self.dispatched_success / self.received_total) * 100

            handler_rate = 0.0
            if self.dispatched_success > 0:
                handler_rate = (self.handler_success / self.dispatched_success) * 100

            # 计算百分位数
            dispatch_p50, dispatch_p95, dispatch_p99 = self._calculate_percentiles(
                self.dispatch_latencies
            )
            handler_p50, handler_p95, handler_p99 = self._calculate_percentiles(
                self.handler_latencies
            )

            # 运行时间
            uptime_seconds = time.time() - self.start_time

            return {
                # 基础计数
                "received_total": self.received_total,
                "dispatched_success": self.dispatched_success,
                "dispatched_failed": self.dispatched_failed,
                "handler_success": self.handler_success,
                "handler_failed": self.handler_failed,
                "dispatch_queue_size": self.dispatch_queue_size,

                # 成功率
                "dispatch_success_rate": f"{dispatch_rate:.2f}%",
                "handler_success_rate": f"{handler_rate:.2f}%",

                # 延迟统计（毫秒）
                "dispatch_latency": {
                    "avg_ms": f"{self._avg_dispatch_latency_ms:.2f}",
                    "p50_ms": f"{dispatch_p50:.2f}",
                    "p95_ms": f"{dispatch_p95:.2f}",
                    "p99_ms": f"{dispatch_p99:.2f}",
                },
                "handler_latency": {
                    "avg_ms": f"{self._avg_handler_latency_ms:.2f}",
                    "p50_ms": f"{handler_p50:.2f}",
                    "p95_ms": f"{handler_p95:.2f}",
                    "p99_ms": f"{handler_p99:.2f}",
                },

                # 吞吐量（每秒）
                "throughput": {
                    "messages_per_second": f"{self.received_total / max(1, uptime_seconds):.2f}",
                    "dispatched_per_second": f"{self.dispatched_success / max(1, uptime_seconds):.2f}",
                },

                # 运行时间
                "uptime_seconds": f"{uptime_seconds:.0f}",
            }

    def _calculate_percentiles(self, data: List[float]) -> tuple:
        """计算百分位数

        Args:
            data: 数据列表

        Returns:
            (p50, p95, p99) 元组
        """
        if not data:
            return (0.0, 0.0, 0.0)

        sorted_data = sorted(data)
        n = len(sorted_data)

        p50_idx = int(n * 0.50)
        p95_idx = int(n * 0.95)
        p99_idx = int(n * 0.99)

        return (
            sorted_data[min(p50_idx, n - 1)],
            sorted_data[min(p95_idx, n - 1)],
            sorted_data[min(p99_idx, n - 1)],
        )

    def reset(self):
        """重置所有指标"""
        with self.lock:
            self.received_total = 0
            self.dispatched_success = 0
            self.dispatched_failed = 0
            self.handler_success = 0
            self.handler_failed = 0
            self.dispatch_queue_size = 0  # ✅ 派发队列大小

            self.dispatch_latencies.clear()
            self.handler_latencies.clear()

            self._avg_dispatch_latency_ms = 0.0
            self._avg_handler_latency_ms = 0.0

            self.start_time = time.time()

    def print_summary(self):
        """打印指标摘要"""
        summary = self.get_summary()

        print("\n" + "=" * 60)
        print("📊 AgentCP 消息处理指标")
        print("=" * 60)

        # 基础计数
        print(f"\n📨 消息统计:")
        print(f"  接收总数: {summary['received_total']}")
        print(f"  派发成功: {summary['dispatched_success']}")
        print(f"  派发失败: {summary['dispatched_failed']}")
        print(f"  处理成功: {summary['handler_success']}")
        print(f"  处理失败: {summary['handler_failed']}")
        print(f"  派发队列: {summary['dispatch_queue_size']}")

        # 成功率
        print(f"\n✅ 成功率:")
        print(f"  派发成功率: {summary['dispatch_success_rate']}")
        print(f"  处理成功率: {summary['handler_success_rate']}")

        # 延迟
        print(f"\n⏱️  延迟统计:")
        print(f"  派发延迟 (ms):")
        print(f"    平均: {summary['dispatch_latency']['avg_ms']}")
        print(f"    P50:  {summary['dispatch_latency']['p50_ms']}")
        print(f"    P95:  {summary['dispatch_latency']['p95_ms']}")
        print(f"    P99:  {summary['dispatch_latency']['p99_ms']}")

        print(f"  处理延迟 (ms):")
        print(f"    平均: {summary['handler_latency']['avg_ms']}")
        print(f"    P50:  {summary['handler_latency']['p50_ms']}")
        print(f"    P95:  {summary['handler_latency']['p95_ms']}")
        print(f"    P99:  {summary['handler_latency']['p99_ms']}")

        # 吞吐量
        print(f"\n🚀 吞吐量:")
        print(f"  接收速率: {summary['throughput']['messages_per_second']} msg/s")
        print(f"  派发速率: {summary['throughput']['dispatched_per_second']} msg/s")

        # 运行时间
        print(f"\n⏰ 运行时间: {summary['uptime_seconds']} 秒")
        print("=" * 60 + "\n")



# -*- coding: utf-8 -*-
"""
统一监控服务

集成滑动窗口统计和时间序列存储，提供完整的监控解决方案
"""

import threading
import time
from typing import Dict, Any, List, Optional

from .sliding_window import SlidingWindowMetrics
from .metrics_store import MetricsStore


class MonitoringService:
    """统一监控服务

    职责：
    - 定期采集指标快照（每10秒）
    - 更新滑动窗口统计
    - 持久化到时间序列存储
    - 提供实时和历史数据查询接口
    """

    def __init__(
        self,
        agent_id: str,
        metrics_collector,
        db_path: str,
        snapshot_interval: int = 10
    ):
        """初始化监控服务

        Args:
            agent_id: AgentID 标识
            metrics_collector: MessageMetrics 实例
            db_path: 时间序列数据库路径
            snapshot_interval: 快照间隔（秒），默认10秒
        """
        self.agent_id = agent_id
        self.metrics_collector = metrics_collector
        self.snapshot_interval = snapshot_interval

        # 初始化组件
        self.sliding_windows = SlidingWindowMetrics()
        self.metrics_store = MetricsStore(db_path)

        # 线程控制
        self._running = False
        self._snapshot_thread: Optional[threading.Thread] = None

        # 统计信息
        self._snapshot_count = 0
        self._last_cleanup_time = time.time()

    def start(self):
        """启动监控服务"""
        if self._running:
            print(f"⚠️ [MonitoringService] 监控服务已在运行")
            return

        self._running = True
        self._snapshot_thread = threading.Thread(
            target=self._snapshot_loop,
            daemon=True,
            name=f"MetricsSnapshot-{self.agent_id}"
        )
        self._snapshot_thread.start()
        print(f"📊 [MonitoringService] 已启动 (agent_id={self.agent_id}, interval={self.snapshot_interval}s)")

    def stop(self, wait: bool = True):
        """停止监控服务

        Args:
            wait: 是否等待线程完全停止（默认True）
                  设为False可避免阻塞主流程
        """
        if not self._running:
            return

        self._running = False

        # 如果需要等待线程停止
        if wait and self._snapshot_thread and self._snapshot_thread.is_alive():
            self._snapshot_thread.join(timeout=5.0)

        # 最后一次快照保存（仅在等待模式下执行）
        if wait:
            try:
                self._take_snapshot()
                print(f"📊 [MonitoringService] 已停止 (共采集 {self._snapshot_count} 次快照)")
            except Exception as e:
                print(f"⚠️ [MonitoringService] 最终快照失败: {e}")
        else:
            print(f"📊 [MonitoringService] 停止信号已发送（非阻塞模式）")

        if wait:
            self._snapshot_thread = None

    def _snapshot_loop(self):
        """快照循环 - 每N秒收集一次数据"""
        while self._running:
            try:
                self._take_snapshot()

                # 定期清理旧数据（每小时一次）
                now = time.time()
                if now - self._last_cleanup_time > 3600:
                    self._cleanup_old_data()
                    self._last_cleanup_time = now

            except Exception as e:
                print(f"❌ [MonitoringService] 快照失败: {e}")
                import traceback
                traceback.print_exc()

            # 等待下一次快照
            time.sleep(self.snapshot_interval)

    def _take_snapshot(self):
        """执行一次快照采集（非阻塞）"""
        try:
            # 1. 获取当前指标（使用 timeout 防止阻塞）
            current_metrics = self.metrics_collector.get_summary()
            current_metrics['agent_id'] = self.agent_id
            current_metrics['timestamp'] = time.time()
        except Exception as e:
            # 获取指标失败，跳过本次快照（不影响核心流程）
            print(f"⚠️ [MonitoringService] 获取指标失败，跳过本次快照: {e}")
            return

        # 解析字符串格式的指标（兼容现有 MessageMetrics）
        self._parse_metrics(current_metrics)

        # 2. 更新滑动窗口
        self.sliding_windows.update(current_metrics)

        # 3. 存储到时间序列数据库
        self.metrics_store.insert_snapshot(current_metrics)

        self._snapshot_count += 1

        # 调试日志（可选）
        if self._snapshot_count % 6 == 0:  # 每1分钟打印一次
            print(
                f"📊 [MonitoringService] 快照 #{self._snapshot_count}: "
                f"received={current_metrics.get('received_total', 0)}, "
                f"queue={current_metrics.get('dispatch_queue_size', 0)}"
            )

    def _parse_metrics(self, metrics: dict):
        """解析和标准化指标数据

        处理现有 MessageMetrics 返回的字符串格式指标
        """
        # 解析延迟数据（从嵌套字典中提取）
        if 'dispatch_latency' in metrics and isinstance(metrics['dispatch_latency'], dict):
            dispatch_latency = metrics['dispatch_latency']
            metrics['avg_dispatch_latency_ms'] = self._parse_float(dispatch_latency.get('avg_ms', '0'))
            metrics['p50_dispatch_latency_ms'] = self._parse_float(dispatch_latency.get('p50_ms', '0'))
            metrics['p95_dispatch_latency_ms'] = self._parse_float(dispatch_latency.get('p95_ms', '0'))
            metrics['p99_dispatch_latency_ms'] = self._parse_float(dispatch_latency.get('p99_ms', '0'))

        if 'handler_latency' in metrics and isinstance(metrics['handler_latency'], dict):
            handler_latency = metrics['handler_latency']
            metrics['avg_handler_latency_ms'] = self._parse_float(handler_latency.get('avg_ms', '0'))

        # 解析运行时间
        if 'uptime_seconds' in metrics:
            metrics['uptime_seconds'] = self._parse_float(metrics['uptime_seconds'])

    def _parse_float(self, value) -> float:
        """安全地解析浮点数"""
        if isinstance(value, (int, float)):
            return float(value)
        if isinstance(value, str):
            try:
                return float(value)
            except ValueError:
                return 0.0
        return 0.0

    def _cleanup_old_data(self):
        """清理旧数据"""
        try:
            self.metrics_store.cleanup_old_data(retention_days=7)
        except Exception as e:
            print(f"⚠️ [MonitoringService] 清理旧数据失败: {e}")

    def get_realtime_metrics(self) -> Dict[str, Any]:
        """获取实时指标（包括所有时间窗口）

        Returns:
            包含以下内容的字典：
            - agent_id: AgentID 标识
            - timestamp: 当前时间戳
            - cumulative: 累计指标（来自 MessageMetrics）
            - windows: 所有时间窗口的统计数据
        """
        base_metrics = self.metrics_collector.get_summary()
        window_stats = self.sliding_windows.get_all_windows()

        return {
            'agent_id': self.agent_id,
            'timestamp': time.time(),
            'cumulative': base_metrics,  # 累计指标
            'windows': window_stats,     # 时间窗口指标
        }

    def get_window_metrics(self, window_names: List[str]) -> Dict[str, Dict[str, Any]]:
        """获取指定时间窗口的指标

        Args:
            window_names: 窗口名称列表，如 ['1m', '3m', '5m']

        Returns:
            窗口统计数据字典
        """
        all_windows = self.sliding_windows.get_all_windows()
        return {
            name: all_windows.get(name, {})
            for name in window_names
            if name in all_windows
        }

    def get_history(self, from_ts: int, to_ts: int, limit: int = 1000) -> List[Dict[str, Any]]:
        """获取历史数据

        Args:
            from_ts: 起始时间戳
            to_ts: 结束时间戳
            limit: 最大返回记录数

        Returns:
            历史数据列表
        """
        return self.metrics_store.query_range(from_ts, to_ts, self.agent_id, limit)

    def get_latest_history(self, limit: int = 100) -> List[Dict[str, Any]]:
        """获取最新的历史数据

        Args:
            limit: 最大返回记录数

        Returns:
            最新的历史数据列表
        """
        return self.metrics_store.query_latest(self.agent_id, limit)

    def get_service_info(self) -> Dict[str, Any]:
        """获取监控服务信息

        Returns:
            包含服务状态和统计信息的字典
        """
        store_stats = self.metrics_store.get_stats()

        return {
            'agent_id': self.agent_id,
            'running': self._running,
            'snapshot_interval': self.snapshot_interval,
            'snapshot_count': self._snapshot_count,
            'store_stats': store_stats,
        }

    def reset_windows(self):
        """重置所有时间窗口（用于测试）"""
        self.sliding_windows.reset()



# -*- coding: utf-8 -*-
"""
独立监控数据读取器

支持跨进程读取监控数据，无需依赖全局变量或进程内通信
"""

import os
import time
from typing import Dict, Any, Optional, List

from .metrics_store import MetricsStore
from .sliding_window import SlidingWindowMetrics


class StandaloneMonitoringReader:
    """独立监控数据读取器

    可以在任何进程中使用，通过直接读取 SQLite 数据库获取监控数据
    """

    def __init__(self, db_path: str = None):
        """初始化读取器

        Args:
            db_path: 数据库路径，如果为空则使用默认路径
        """
        if db_path is None:
            # 使用默认路径
            project_root = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..'))
            db_path = os.path.join(project_root, 'backup', 'metrics_timeseries.db')

        self.db_path = db_path
        self.metrics_store = MetricsStore(db_path)

    def get_realtime_metrics(self) -> Dict[str, Any]:
        """获取实时监控指标

        Returns:
            包含累计指标和时间窗口统计的字典
        """
        # 检查数据库文件是否存在
        if not os.path.exists(self.db_path):
            raise FileNotFoundError(f"监控数据库不存在: {self.db_path}")

        # 获取最新的一条记录作为累计指标
        latest_records = self.metrics_store.query_latest(limit=1)
        if not latest_records:
            raise ValueError("数据库中没有监控数据")

        cumulative = latest_records[0]

        # 计算时间窗口统计
        windows = self._calculate_windows()

        return {
            'agent_id': cumulative.get('agent_id', 'unknown'),
            'timestamp': cumulative.get('timestamp', time.time()),
            'cumulative': self._format_cumulative(cumulative),
            'windows': windows
        }

    def _format_cumulative(self, record: dict) -> dict:
        """格式化累计指标"""
        # 计算成功率
        received_total = record.get('received_total', 0)
        dispatched_success = record.get('dispatched_success', 0)

        dispatch_success_rate = "0.00%"
        if received_total > 0:
            rate = (dispatched_success / received_total) * 100
            dispatch_success_rate = f"{rate:.2f}%"

        # 计算运行时长（使用最早的记录时间）
        store_stats = self.metrics_store.get_stats()
        min_ts = store_stats.get('min_timestamp', record['timestamp'])
        uptime_seconds = record['timestamp'] - min_ts if min_ts else 0

        return {
            'received_total': record.get('received_total', 0),
            'dispatched_success': record.get('dispatched_success', 0),
            'dispatched_failed': record.get('dispatched_failed', 0),
            'handler_success': record.get('handler_success', 0),
            'handler_failed': record.get('handler_failed', 0),
            'dispatch_queue_size': record.get('dispatch_queue_size', 0),
            'dispatch_success_rate': dispatch_success_rate,
            'uptime_seconds': str(int(uptime_seconds)),
            'dispatch_latency': {
                'avg_ms': f"{record.get('avg_dispatch_latency_ms', 0):.2f}",
                'p50_ms': f"{record.get('p50_dispatch_latency_ms', 0):.2f}",
                'p95_ms': f"{record.get('p95_dispatch_latency_ms', 0):.2f}",
                'p99_ms': f"{record.get('p99_dispatch_latency_ms', 0):.2f}",
            }
        }

    def _calculate_windows(self) -> Dict[str, Dict[str, Any]]:
        """计算时间窗口统计"""
        now = int(time.time())

        windows = {
            '1m': self._calculate_window(now, 60),
            '3m': self._calculate_window(now, 180),
            '5m': self._calculate_window(now, 300),
            '10m': self._calculate_window(now, 600),
            '15m': self._calculate_window(now, 900),
        }

        return windows

    def _calculate_window(self, now: int, duration: int) -> Dict[str, Any]:
        """计算单个时间窗口的统计数据"""
        from_ts = now - duration
        to_ts = now

        # 查询时间范围内的数据
        records = self.metrics_store.query_range(from_ts, to_ts)

        if len(records) < 2:
            # 数据不足，返回空统计
            return {
                'throughput_per_second': 0.0,
                'avg_latency_ms': 0.0,
                'success_rate': 0.0,
                'total_messages': 0,
                'failed_messages': 0,
                'avg_queue_size': 0.0,
                'window_duration': duration,
                'data_points_count': len(records)
            }

        # 计算增量
        first_record = records[0]
        last_record = records[-1]

        received_delta = last_record['received_total'] - first_record['received_total']
        success_delta = last_record['dispatched_success'] - first_record['dispatched_success']
        failed_delta = last_record['dispatched_failed'] - first_record['dispatched_failed']

        # 计算实际时间跨度
        actual_duration = last_record['timestamp'] - first_record['timestamp']
        if actual_duration < 1:
            actual_duration = 1

        # 计算吞吐量
        throughput = received_delta / actual_duration

        # 计算成功率
        success_rate = 0.0
        if received_delta > 0:
            success_rate = (success_delta / received_delta) * 100

        # 计算平均延迟
        latencies = [r['avg_dispatch_latency_ms'] for r in records if r['avg_dispatch_latency_ms'] > 0]
        avg_latency = sum(latencies) / len(latencies) if latencies else 0.0

        # 计算平均队列大小
        queue_sizes = [r['dispatch_queue_size'] for r in records]
        avg_queue_size = sum(queue_sizes) / len(queue_sizes) if queue_sizes else 0.0

        return {
            'throughput_per_second': round(throughput, 2),
            'avg_latency_ms': round(avg_latency, 2),
            'success_rate': round(success_rate, 2),
            'total_messages': received_delta,
            'failed_messages': failed_delta,
            'avg_queue_size': round(avg_queue_size, 1),
            'window_duration': duration,
            'data_points_count': len(records)
        }

    def get_window_metrics(self, window_names: List[str]) -> Dict[str, Dict[str, Any]]:
        """获取指定时间窗口的指标"""
        now = int(time.time())

        window_durations = {
            '1m': 60,
            '3m': 180,
            '5m': 300,
            '10m': 600,
            '15m': 900,
        }

        result = {}
        for name in window_names:
            if name in window_durations:
                result[name] = self._calculate_window(now, window_durations[name])

        return result

    def get_history(self, from_ts: int, to_ts: int, limit: int = 1000) -> List[Dict[str, Any]]:
        """获取历史数据"""
        return self.metrics_store.query_range(from_ts, to_ts, limit=limit)

    def get_latest_history(self, limit: int = 100) -> List[Dict[str, Any]]:
        """获取最新的历史数据"""
        return self.metrics_store.query_latest(limit=limit)

    def get_service_info(self) -> Dict[str, Any]:
        """获取监控服务信息"""
        store_stats = self.metrics_store.get_stats()

        # 检查是否有最近的数据更新（最近30秒内）
        max_ts = store_stats.get('max_timestamp', 0)
        is_running = (time.time() - max_ts) < 30 if max_ts else False

        return {
            'agent_id': 'unknown',
            'running': is_running,
            'snapshot_interval': 10,
            'snapshot_count': store_stats.get('total_records', 0),
            'store_stats': store_stats
        }


def get_standalone_reader(db_path: str = None) -> StandaloneMonitoringReader:
    """获取独立监控读取器实例

    Args:
        db_path: 数据库路径，如果为空则使用默认路径

    Returns:
        StandaloneMonitoringReader 实例
    """
    return StandaloneMonitoringReader(db_path)
